<?php
/**
 * Slope functions and definitions
 *
 * @link https://developer.wordpress.org/themes/basics/theme-functions/
 *
 * @package Slope
 */

/**
 * The current version of the theme.
 */
define( 'SLOPE_VERSION', '1.0.2' );

if ( ! function_exists( 'slope_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function runs before the init hook. The init hook is too late
	 * for some features, such as indicating support for post thumbnails.
	 */
	function slope_setup() {
		/*
		 * Make theme available for translation.
		 * Translations can be filed in the /languages/ directory.
		 * If you're building a theme based on Slope, use a find and replace
		 * to change 'slope' to the name of your theme in all the template files.
		 */
		load_theme_textdomain( 'slope', get_template_directory() . '/languages' );

		global $slope_settings;
		$slope_settings = new SLOPE_Settings();

		$google_request = str_replace( ',', '%2C', slope_get_google_font_uri() );

		// This theme styles the visual editor with editor-style.css to match the theme style.
		add_editor_style( $google_request );

		// Add default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		/*
		 * Let WordPress manage the document title.
		 * By adding theme support, we declare that this theme does not use a
		 * hard-coded <title> tag in the document head, and expect WordPress to
		 * provide it for us.
		 */
		add_theme_support( 'title-tag' );

		/*
		 * Enable support for Post Thumbnails on posts and pages.
		 *
		 * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
		 */
		add_theme_support( 'post-thumbnails' );

		// This theme uses wp_nav_menu() in one location.
		register_nav_menus(
			array(
				'menu-1' => esc_html__( 'Primary', 'slope' ),
				'menu-2' => esc_html__( 'Footer', 'slope' ),
			)
		);

		/*
		 * Switch default core markup for search form, comment form, and comments
		 * to output valid HTML5.
		 */
		add_theme_support(
			'html5', array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
			)
		);

		// Set up the WordPress core custom background feature.
		add_theme_support(
			'custom-background', apply_filters(
				'slope_custom_background_args', array(
					'default-color' => 'ffffff',
					'default-image' => '',
				)
			)
		);

		// Add theme support for selective refresh for widgets.
		add_theme_support( 'customize-selective-refresh-widgets' );

		// Enable support for site logo.
		add_theme_support(
			'custom-logo', array(
				'height'      => 250,
				'width'       => 250,
				'flex-width'  => true,
				'flex-height' => true,
			)
		);

		// Add WooCommerce support.
		add_theme_support( 'woocommerce' );
		add_theme_support( 'wc-product-gallery-zoom' );
		add_theme_support( 'wc-product-gallery-lightbox' );
		add_theme_support( 'wc-product-gallery-slider' );
	}
endif;
add_action( 'after_setup_theme', 'slope_setup' );

/**
 * Set the content width in pixels, based on the theme's design and stylesheet.
 *
 * Priority 0 to make it available to lower priority callbacks.
 *
 * @global int $content_width
 */
function slope_content_width() {
	$GLOBALS['content_width'] = apply_filters( 'slope_content_width', 750 );
}
add_action( 'after_setup_theme', 'slope_content_width', 0 );

/**
 * Register widget area.
 *
 * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
 */
function slope_widgets_init() {
	register_sidebar(
		array(
			'name'          => esc_html__( 'Footer Sidebar', 'slope' ),
			'id'            => 'sidebar-1',
			'description'   => esc_html__( 'Add widgets here.', 'slope' ),
			'before_widget' => '<section id="%1$s" class="widget grid__col grid__col--1-of-4 %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h2 class="widget-title">',
			'after_title'   => '</h2>',
		)
	);
	register_sidebar(
		array(
			'name'          => esc_html__( 'Single Post', 'slope' ),
			'id'            => 'sidebar-2',
			'description'   => esc_html__( 'Add widgets here to show on Single Posts.', 'slope' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h2 class="widget-title">',
			'after_title'   => '</h2>',
		)
	);
	register_sidebar(
		array(
			'name'          => esc_html__( 'Single Page', 'slope' ),
			'id'            => 'sidebar-3',
			'description'   => esc_html__( 'Add widgets here to show on Single Pages.', 'slope' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h2 class="widget-title">',
			'after_title'   => '</h2>',
		)
	);
	register_sidebar(
		array(
			'name'          => esc_html__( 'Blog Sidebar', 'slope' ),
			'id'            => 'sidebar-5',
			'description'   => esc_html__( 'Add widgets here to show on Blog pages.', 'slope' ),
			'before_widget' => '<section id="%1$s" class="widget %2$s">',
			'after_widget'  => '</section>',
			'before_title'  => '<h2 class="widget-title">',
			'after_title'   => '</h2>',
		)
	);

	if ( function_exists( 'is_woocommerce' ) ) :
		register_sidebar(
			array(
				'name'          => esc_html__( 'Shop', 'slope' ),
				'id'            => 'sidebar-4',
				'description'   => esc_html__( 'Add widgets here to show on Shop archives.', 'slope' ),
				'before_widget' => '<section id="%1$s" class="widget %2$s">',
				'after_widget'  => '</section>',
				'before_title'  => '<h2 class="widget-title">',
				'after_title'   => '</h2>',
			)
		);
	endif;
}
add_action( 'widgets_init', 'slope_widgets_init' );

/**
 * Enqueue scripts and styles.
 */
function slope_scripts() {
	// Google fonts.
	$google_request = slope_get_google_font_uri();

	if ( '' !== $google_request ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'slope-google-fonts',
			$google_request,
			'',
			SLOPE_VERSION
		);
	}

	wp_enqueue_style( 'slope-style', get_stylesheet_uri(), array( 'dashicons' ) );

	// Theme option styles
	// Note: in the Customizer, these rules are output inline in the document head,
	// because Ajax calls won't recognize the preview values of settings.
	$version = absint( get_theme_mod( 'version', 0 ) );
	if ( ! is_customize_preview() ) {
		wp_enqueue_style(
			'slope-dynamic-style',
			add_query_arg( 'action', 'slope-css-rules', admin_url( 'admin-ajax.php' ) ),
			array( 'slope-style' ),
			$version,
			'screen'
		);
	}

	wp_enqueue_script( 'slope-navigation', get_template_directory_uri() . '/js/navigation.js', array(), '20151215', true );

	wp_enqueue_script( 'slope-script', get_template_directory_uri() . '/js/frontend.min.js', array( 'jquery', 'imagesloaded' ), SLOPE_VERSION, true );

	wp_enqueue_script( 'salvattore', get_template_directory_uri() . '/js/salvattore.min.js', array(), '1.0.9', true );

	wp_register_script( 'flexslider', get_template_directory_uri() . '/js/flexslider/jquery.flexslider-min.js', array( 'jquery' ), '2.6.3', true );

	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' );
	}
}
add_action( 'wp_enqueue_scripts', 'slope_scripts' );

/**
* Remove Subtitle plugin CSS.
*
* @see https://wordpress.org/plugins/subtitles/
*/
if ( class_exists( 'Subtitles' ) && method_exists( 'Subtitles', 'subtitle_styling' ) ) {
	remove_action( 'wp_head', array( Subtitles::getInstance(), 'subtitle_styling' ) );
}

/**
 * Remove global Subtitle support, hooked in manually for single posts/pages.
 *
 * @return bool
 */
function remove_subtitles_support() {
	return false;
}
add_action( 'subtitle_view_supported', 'remove_subtitles_support' );

if ( ! function_exists( 'slope_required_plugins' ) ) :
	/**
	 * Require plugins with TGMPA library.
	 *
	 * @since 1.0.0
	 * @return void
	 */
	function slope_required_plugins() {
		$plugins = array(
			array(
				'name'             => 'WooCommerce - excelling eCommerce',
				'slug'             => 'woocommerce',
				'required'         => false,
				'force_activation' => false,
			),
			array(
				'name'             => 'StagTools',
				'slug'             => 'stagtools',
				'required'         => false,
				'force_activation' => false,
			),
			array(
				'name'             => 'Jetpack',
				'slug'             => 'jetpack',
				'required'         => false,
				'force_activation' => false,
			),
		);

		tgmpa( $plugins );
	}
endif; // End of slope_required_plugins.
add_action( 'tgmpa_register', 'slope_required_plugins' );

/**
 * Include theme updater files.
 *
 * @return void
 */
function slope_theme_updater() {
	require get_parent_theme_file_path() . '/inc/updater/theme-updater.php';
}
add_action( 'after_setup_theme', 'slope_theme_updater' );

/**
 * Implement the Custom Header feature.
 */
require get_template_directory() . '/inc/custom-header.php';

/**
 * Custom template tags for this theme.
 */
require get_template_directory() . '/inc/template-tags.php';

/**
 * Additional features to allow styling of the templates.
 */
require get_template_directory() . '/inc/template-functions.php';

/**
 * Load Jetpack compatibility file.
 */
require get_template_directory() . '/inc/jetpack.php';

// Load WooCommerce compatibility file.
require get_template_directory() . '/inc/woocommerce.php';

require get_parent_theme_file_path() . '/inc/class-tgm-plugin-activation.php';

/** Standard includes  */
$includes = [
	// Load theme files.
	'/customizer/load.php',
	'/class-slope-page-templates.php',
	'/class-slope-widget.php',
	'/class-slope-widgetized-pages.php',

	// Widgets.
	'widgets/class-slope-widget-category-fold.php',
	'widgets/class-slope-widget-contributors.php',
	'widgets/class-slope-widget-cta.php',
	'widgets/class-slope-widget-feature-callout.php',
	'widgets/class-slope-widget-features.php',
	'widgets/class-slope-widget-images.php',
	'widgets/class-slope-widget-post-slider.php',
	'widgets/class-slope-widget-posts.php',
	'widgets/class-slope-widget-products.php',
	'widgets/class-slope-widget-static-content.php',
	'widgets/class-slope-widget-testimonials.php',
];
foreach ( $includes as $file ) {
	require get_template_directory() . '/inc/' . $file;
}
