<?php
/**
 * Automatically register widgets areas for pages using Widgetized template.
 *
 * @package Slope
 */

/**
 * Manage widgetized page templates.
 *
 * @package Slope
 * @category Page Template
 * @author Codestag
 */
class Slope_Widgetized_Pages {

	/**
	 * Cache found pages.
	 *
	 * @var string
	 */
	private $transient;

	/**
	 * Hook in to WordPress
	 */
	public function __construct() {
		$this->transient = 'slope_widgetized_pages';

		add_action( 'save_post', array( $this, 'flush_cache' ) );
		add_action( 'widgets_init', array( $this, 'register_widget_areas' ), 99 );
	}

	/**
	 * Create a widgetized sidebar for each page template.
	 */
	public function register_widget_areas() {
		$pages = $this->get_pages();

		if ( empty( $pages ) ) {
			return;
		}

		foreach ( $pages as $page ) {
			register_sidebar( array(
				// Translators: %s Page title for widgetized page.
				'name'          => sprintf( __( 'Page: %s', 'slope' ), get_the_title( $page ) ),
				// Translators: %s Page title for widgetized page.
				'description'   => sprintf( __( 'Widgets that appear on the "%s" page.', 'slope' ), get_the_title( $page ) ),
				'id'            => 'widget-area-page-' . $page,
				'before_widget' => '<aside id="%1$s" class="page-widget home-widget %2$s">',
				'after_widget'  => '</aside>',
				'before_title'  => '<h2 class="home-widget-title">',
				'after_title'   => '</h2>',
			) );
		}
	}

	/**
	 * Find all pages with a widgetized page template.
	 *
	 * @return array $pages
	 */
	public function get_pages() {
		$pages = get_transient( $this->transient );

		if ( false === $pages ) {
			$pages = array();

			$query = new WP_Query( array(
				'fields'     => 'ids',
				'nopaging'   => true,
				'post_type'  => 'page',
				'meta_query' => array(
					array(
						'key' => '_wp_page_template',
						'value' => array(
							'page-templates/template-widgetized.php',
						),
						'compare' => 'IN',
					),
				),
			) );

			if ( ! empty( $query->posts ) ) {
				$pages = $query->posts;
			}

			set_transient( $this->transient, $pages );
		}

		return $pages;
	}

	/**
	 * Clear cache when a page is updated.
	 */
	public function flush_cache() {
		delete_transient( $this->transient );
	}
}

new Slope_Widgetized_Pages();
