<?php
/**
 * Dynamic CSS rules based on customizer settings.
 *
 * @package Slope
 */

/**
 * Return an object containing an array of CSS rules.
 *
 * This function behaves differently depending on which parameters are given.
 * - With no parameters, it returns the rules for all the CSS-related settings using their current stored values.
 * - With an array of setting IDs, it returns the rules for those specific settings, using their current stored values.
 * - With both an array of setting IDs and a corresponding array of values, it will return the rules for those specific settings, using the given values (sanitized).
 *
 * @since 1.0.0.
 *
 * @param  array $setting_ids       An optional array of setting IDs.
 * @param  array $preview_values    An optional array of values for the settings.
 *
 * @return mixed|SLOPE_CSS|void
 */
function slope_get_css_rules( $setting_ids = array(), $preview_values = array() ) {
	$settings      = new SLOPE_Settings();
	$css           = new SLOPE_CSS();
	$setting_pairs = array();

	// Default array of settings.
	if ( empty( $setting_ids ) ) {
		$setting_ids = array(
			'background_color',
			'color-accent',
			'color-footer-background',
			'color-footer-text',
			'color-secondary',
			'font-body',
			'font-headers',
			'bool-hide-credit',
			'custom-settings',
			'site-width',
			'header-text-color',
		);
	}

	// Use the saved setting values if no preview values are given.
	if ( empty( $preview_values ) ) {
		foreach ( (array) $setting_ids as $id ) {
			$setting_pairs[ $id ] = $settings->get_value( $id );
		}
	} else {
		$setting_pairs = array_combine( (array) $setting_ids, (array) $preview_values );
		array_walk( $setting_pairs, array( $settings, 'sanitize_value' ) );
	}

	// Iterate through each setting.
	foreach ( $setting_pairs as $setting_id => $sanitized_value ) {
		/**
		 * Filter to override the CSS rules for a particular setting.
		 *
		 * To override the CSS rules for a setting, create a function that accepts all three
		 * of the parameters of this filter. Use the SLOPE_CSS methods in the $css parameter
		 * to add custom rules, then return $css instead of $override.
		 *
		 * Example:
		 * function child_css_override_titlecolor( $override, $setting_id, $css ) {
		 *     if ( 'color-site-title' === $setting_id ) {
		 *         $css->add( array(
		 *             'selectors'    => array(
		 *                 '.site-title', '.site-title a',
		 *             ),
		 *             'declarations' => array(
		 *                 'background-color' => $settings->get_value( $setting_id ),
		 *             ),
		 *         ) );
		 *         return $css;
		 *     }
		 *
		 *     return $override;
		 * }
		 * add_filter( 'slope_css_rule_override', 'child_css_override_titlecolor', 10, 4 );
		 *
		 * To add rules for a setting not in the defaults list, or a new setting, use the 'custom-settings'
		 * value for the setting ID, like this:
		 *
		 *     if ( 'custom-settings' === $setting_id ) {
		 *         // Rules go here.
		 *     }
		 *
		 * @since 1.0.0.
		 *
		 * @param bool|SLOPE_CSS    $override           Value determines whether setting CSS rules get overridden.
		 * @param string               $setting_id         The ID of the setting to override.
		 * @param mixed                $sanitized_value    The sanitized value currently assigned to the setting.
		 * @param SLOPE_CSS         $css                The instance of SLOPE_CSS.
		 */
		$override = apply_filters( 'slope_css_rule_override', false, $setting_id, $sanitized_value, $css );
		if ( false !== $override && 'SLOPE_CSS' === get_class( $override ) ) {
			$css = $override;
			continue;
		}

		switch ( $setting_id ) {
			case 'font-body':
				$css->add( array(
					'selectors'    => array(
						'body',
						'input',
						'select',
						'textarea',
						'.entry-subtitle',
					),
					'declarations' => array(
						'font-family' => "'$sanitized_value'",
					),
				) );

				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'--slope-font-body' => "'$sanitized_value'",
					),
				) );
				break;
			case 'font-headers':
				$css->add( array(
					'selectors'    => array(
						'button',
						'.button',
						'label',
						'.site-title',
						'.site-info-footer',
						'.entry-meta',
						'.comment-meta',
						'.comment-metadata',
						'.comment-reply-link',
						'.post-navigation',
						'input[type=submit]',
						'input[type=button]',
						'input[type=reset]',
						'.main-navigation',
						'h1,h2,h3,h4,h5,h6',
						'.category-fold-posts .entry-date',
						'table th',
						'table thead',
						'.mobile-navigation',
						'.contributor-archive-link',
						'.slope_testimonials cite',
						'.slope_images figcaption',
						'.widget_recent_entries .post-date',
						'.content-pagination',
						'.footer-meta',
						'div.sharedaddy h3.sd-title',
						'.woocommerce .price',
						'.woocommerce-pagination',
						'.woocommerce span.onsale',
						'.woocommerce-review-link',
						'.woocommerce-result-count',
						'.woocommerce .product_meta',
						'.woocommerce-review__author',
						'.woocommerce-ordering .orderby',
						'.woocommerce-Reviews .comment-reply-title',
						'.woocommerce div.product .woocommerce-tabs ul.tabs',
						'.site-header-cart .count',
						'.woocommerce-MyAccount-navigation ul',
						'.woocommerce a.added_to_cart',
						'.woocommerce-store-notice',
						'p.demo_store',
						'.woocommerce-error',
						'.woocommerce-info',
						'.woocommerce-message',
					),
					'declarations' => array(
						'font-family' => "'$sanitized_value'",
					),
				) );

				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'--slope-font-header' => "'$sanitized_value'",
					),
				) );
				break;
			case 'color-accent':
				$css->add( array(
					'selectors'    => array(
						'a',
						'.woocommerce-info a',
						'.woo-mobile-cart .count',
						'.woocommerce .star-rating span::before',
					),
					'declarations' => array(
						'color' => $sanitized_value,
					),
				) );

				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'--slope-color-accent' => "$sanitized_value",
					),
				) );

				$css->add( array(
					'selectors'    => array(
						'button',
						'.button',
						'input[type="button"]',
						'input[type="reset"]',
						'input[type="submit"]',
						'.mobile-menu a:hover',
						'.slope_images figcaption',
						'.highlight-heading:before',
						'.content-pagination .page-numbers.current, .content-pagination .page-numbers:hover:not(.dots)',
						'.author-info .all-posts:before',
						'.woocommerce a.button',
						'.woocommerce button.button',
						'.woocommerce input.button',
						'.woocommerce #respond input#submit',
						'.woocommerce #respond input#submit:hover',
						'.woocommerce a.button:hover',
						'.woocommerce button.button:hover',
						'.woocommerce input.button:hover',
						'.woocommerce #respond input#submit.alt',
						'.woocommerce a.button.alt',
						'.woocommerce button.button.alt',
						'.woocommerce input.button.alt',
						'.woocommerce #respond input#submit.alt:hover',
						'.woocommerce a.button.alt:hover',
						'.woocommerce button.button.alt:hover',
						'.woocommerce input.button.alt:hover',
						'.woocommerce #respond input#submit.alt.disabled',
						'.woocommerce #respond input#submit.alt.disabled:hover',
						'.woocommerce #respond input#submit.alt:disabled',
						'.woocommerce #respond input#submit.alt:disabled:hover',
						'.woocommerce #respond input#submit.alt:disabled[disabled]',
						'.woocommerce #respond input#submit.alt:disabled[disabled]:hover',
						'.woocommerce a.button.alt.disabled',
						'.woocommerce a.button.alt.disabled:hover',
						'.woocommerce a.button.alt:disabled',
						'.woocommerce a.button.alt:disabled:hover',
						'.woocommerce a.button.alt:disabled[disabled]',
						'.woocommerce a.button.alt:disabled[disabled]:hover',
						'.woocommerce button.button.alt.disabled',
						'.woocommerce button.button.alt.disabled:hover',
						'.woocommerce button.button.alt:disabled',
						'.woocommerce button.button.alt:disabled:hover',
						'.woocommerce button.button.alt:disabled[disabled]',
						'.woocommerce button.button.alt:disabled[disabled]:hover',
						'.woocommerce input.button.alt.disabled',
						'.woocommerce input.button.alt.disabled:hover',
						'.woocommerce input.button.alt:disabled',
						'.woocommerce input.button.alt:disabled:hover',
						'.woocommerce input.button.alt:disabled[disabled]',
						'.woocommerce input.button.alt:disabled[disabled]:hover',
						'.woocommerce-store-notice',
						'p.demo_store',
						'.woocommerce-message',
					),
					'declarations' => array(
						'background-color' => $sanitized_value,
					),
				) );

				$css->add( array(
					'selectors' => array(
						'.site-info a',
						'.comment-reply-link',
						'.primary-menu > li:hover',
						'.primary-menu li.current-menu-item',
						'.woo-mobile-cart .cart-link',
						'.page-widget .category-link:hover',
					),
					'declarations' => array(
						'border-color' => $sanitized_value,
					),
				) );

				$css->add( array(
					'selectors' => array(
						'button',
						'.button',
						'.background-accent',
						'input[type="button"]',
						'input[type="reset"]',
						'input[type="submit"]',
						'.widget-title:before',
						'.home-widget-title:before',
						'.slope_testimonials a.flex-active',
						'.flex-direction-nav a:hover .background-secondary',
					),
					'declarations' => array(
						'background-color' => $sanitized_value,
					),
				) );

				break;
			case 'color-footer-background':
				$css->add( array(
					'selectors'    => array(
						'.widget-area--footer',
					),
					'declarations' => array(
						'background-color' => $sanitized_value,
					),
				) );

				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'--slope-background-footer' => "$sanitized_value",
					),
				) );
				break;
			case 'color-footer-text':
				$css->add( array(
					'selectors'    => array(
						'.widget-area--footer',
					),
					'declarations' => array(
						'color' => $sanitized_value,
					),
				) );

				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'--slope-color-footer' => "$sanitized_value",
					),
				) );
				break;
			case 'color-secondary':
				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'--slope-color-secondary' => "$sanitized_value",
					),
				) );
				$css->add( array(
					'selectors'    => array(
						'.entry-author',
						'.background-secondary',
						'.woocommerce-products-header',
						'.woocommerce-info',
						'.woocommerce-error',
					),
					'declarations' => array(
						'background-color' => $sanitized_value,
					),
				) );
				break;
			case 'bool-hide-credit':
				if ( true === $sanitized_value ) {
					$css->add( array(
						'selectors'    => array(
							'.site-byline',
						),
						'declarations' => array(
							'display' => 'none',
						),
					) );
				} else {
					$css->add( array(
						'selectors'    => array(
							'.site-byline',
						),
						'declarations' => array(
							'display' => 'block',
						),
					) );
				}
				break;

			case 'site-width':
				$css->add( array(
					'selectors'    => array(
						'body',
					),
					'declarations' => array(
						'--slope-site-width' => "{$sanitized_value}px",
					),
				) );
				$css->add( array(
					'selectors'    => array(
						'.site',
					),
					'declarations' => array(
						'max-width' => "{$sanitized_value}px",
					),
				) );
				break;

			case 'header-text-color':
				$sanitized_value = get_theme_mod( 'header_textcolor', '#ffffff' );

				$css->add( array(
					'selectors'    => array(
						'.custom-header',
					),
					'declarations' => array(
						'color' => "#" . $sanitized_value,
					),
				) );
				break;
		} // End switch().
	} // End foreach().

	return $css;
}

/**
 * Output the theme CSS rules as an inline style element.
 *
 * Used in the Customizer preview pane.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function slope_inline_css_rules() {
	// Get the rules.
	$css = slope_get_css_rules();

	// Build the rules.
	$output = $css->build();

	// Output the rules.
	if ( ! empty( $output ) ) {
		echo "\n<style type=\"text/css\" id=\"slope-inline-css\">\n";
		echo $output; // WPCS: XSS ok.
		echo "\n</style>\n";
	}
}

if ( is_customize_preview() ) {
	// Only add the inline rules in the Customizer preview pane.
	add_action( 'wp_head', 'slope_inline_css_rules', 99 );
}

/**
 * Add the theme CSS rules to the content editor.
 *
 * @since 1.0.0.
 *
 * @param  string $stylesheets The comma-separated string of stylesheet URLs.
 *
 * @return string               The modified string of stylesheet URLs.
 */
function slope_editor_css_rules( $stylesheets ) {
	$stylesheets .= ',' . add_query_arg( 'action', 'slope-css-rules', admin_url( 'admin-ajax.php' ) );
	return $stylesheets;
}

add_filter( 'mce_css', 'slope_editor_css_rules', 99 );

/**
 * Generates the theme CSS as an Ajax response.
 *
 * @since  1.0.0.
 *
 * @return void
 */
function slope_ajax_css_rules() {
	// Make sure this is an Ajax request.
	if ( ! defined( 'DOING_AJAX' ) || true !== DOING_AJAX ) {
		return;
	}

	/**
	 * Filter whether the dynamic stylesheet will send headers telling the browser
	 * to cache the request. Set to false to turn off these headers.
	 *
	 * @since 1.0.0.
	 *
	 * @param bool    $cache_headers
	 */
	if ( true === apply_filters( 'slope_stylesheet_cache_headers', true ) ) {
		/**
		 * Set headers for caching.
		 *
		 * @link http://stackoverflow.com/a/15000868
		 * @link http://www.mobify.com/blog/beginners-guide-to-http-cache-headers/
		 */
		$expires = HOUR_IN_SECONDS;
		header( 'Pragma: public' );
		header( 'Cache-Control: private, max-age=' . $expires );
		header( 'Expires: ' . gmdate( 'D, d M Y H:i:s', time() + $expires ) . ' GMT' );
	}

	// Set header for content type.
	header( 'Content-type: text/css' );

	// Get the rules.
	$css = slope_get_css_rules();

	// Echo the rules.
	echo $css->build(); // WPCS: XSS OK.

	// End the Ajax response.
	die();
}

add_action( 'wp_ajax_slope-css-rules', 'slope_ajax_css_rules' );
add_action( 'wp_ajax_nopriv_slope-css-rules', 'slope_ajax_css_rules' );

/**
 * Returns the theme CSS as a JSON object.
 *
 * Used for asynchronous previewing in the Customizer.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function slope_ajax_css_json() {
	// Make sure this is an Ajax request.
	if ( ! defined( 'DOING_AJAX' ) || true !== DOING_AJAX ) {
		return;
	}

	$settings = new SLOPE_Settings();

	// Get post parameters.
	$setting_ids = array();
	if ( isset( $_POST['sid'] ) ) { // WPCS: CSRF ok.
		$setting_ids = array_map( 'sanitize_title_with_dashes', (array) $_POST['sid'] ); // WPCS: sanitization ok.
	}
	$values = array();
	if ( isset( $_POST['val'] ) ) { // WPCS: CSRF ok.
		foreach ( (array) $_POST['val'] as $index => $raw_value ) { // WPCS: sanitization ok, CSRF ok.
			$values[ $index ] = $settings->sanitize_value( $raw_value, $setting_ids[ $index ] );
		}
	}

	// Generate the rules.
	$css = slope_get_css_rules( $setting_ids, $values );

	// Send response.
	if ( isset( $css->data ) && is_array( $css->data ) && ! empty( $css->data ) ) {
		wp_send_json_success( $css->data );
	} else {
		wp_send_json_error( $css );
	}
}

add_action( 'wp_ajax_slope-css-json', 'slope_ajax_css_json' );

/**
 * Saves a version number (Unix timestamp) when Customizer settings are saved.
 *
 * @since 1.0.0.
 *
 * @return void
 */
function slope_customize_save_version() {
	set_theme_mod( 'version', time() );
}

add_action( 'customize_save_after', 'slope_customize_save_version' );
