<?php
/**
 * Functions which enhance the theme by hooking into WordPress
 *
 * @package Slope
 */

/**
 * Adds custom classes to the array of body classes.
 *
 * @param array $classes Classes for the body element.
 * @return array
 */
function slope_body_classes( $classes ) {
	// Adds a class of group-blog to blogs with more than 1 published author.
	if ( is_multi_author() ) {
		$classes[] = 'group-blog';
	}

	// Adds a class of hfeed to non-singular pages.
	if ( ! is_singular() ) {
		$classes[] = 'hfeed';
	}

	// Adds a class of sidebar layout for single posts.
	if ( is_single() && 'post' === get_post_type() ) {
		$sidebar_position = slope_get_thememod_value( 'single-sidebar' );

		if ( ! is_active_sidebar( 'sidebar-2' ) ) {
			$sidebar_position = 'none';
		}

		$classes[] = 'sidebar-' . $sidebar_position;

		if ( has_post_thumbnail() || '' !== slope_blog_fallback_image() ) {
			$classes[] = 'has-post-thumbnail';
		}
	}

	if ( class_exists( 'woocommerce' ) ) {
		if ( is_shop() || is_post_type_archive( 'product' ) || is_tax( get_object_taxonomies( 'product' ) ) ) {
			$columns  = (int) slope_get_thememod_value( 'shop-column' );
			$position = slope_get_thememod_value( 'shop-sidebar-position' );

			if ( 2 === $columns ) {
				$classes[] = 'sidebar-' . $position;
			}
		}
	}

	if ( is_single() && 'jetpack-testimonial' === get_post_type() && has_post_thumbnail() ) {
		$classes[] = 'has-post-thumbnail';
	}

	// Adds a class of sidebar layout for single pages.
	if ( is_page() && '' === get_page_template_slug() && 'page' === get_post_type() ) {
		if ( has_post_thumbnail() || '' !== slope_blog_fallback_image() ) {
			$classes[] = 'has-post-thumbnail';
		}

		if ( ! slope_is_woocommerce_page() ) {
			$sidebar_position = slope_get_thememod_value( 'page-sidebar' );
			if ( ! is_active_sidebar( 'sidebar-3' ) ) {
				$sidebar_position = 'none';
			}
			$classes[] = 'sidebar-' . $sidebar_position;
		}
	}

	// Adds a class for category page if it has a category thumbnail.
	$fallback_image = slope_blog_fallback_image();
	if ( ( is_archive() || is_search() ) && false !== $fallback_image ) {
		$classes[] = 'has-category-thumb';
	}

	if ( ( is_front_page() || is_home() ) && false !== $fallback_image ) {
		$classes[] = 'has-category-thumb';
	}

	return $classes;
}
add_filter( 'body_class', 'slope_body_classes' );

/**
 * Add a pingback url auto-discovery header for singularly identifiable articles.
 */
function slope_pingback_header() {
	if ( is_singular() && pings_open() ) {
		echo '<link rel="pingback" href="', esc_url( get_bloginfo( 'pingback_url' ) ), '">';
	}
}
add_action( 'wp_head', 'slope_pingback_header' );

/**
 * Custom excerpt text.
 *
 * @param  string $more Default excerpt text.
 * @return string
 */
function slope_excerpt_more( $more ) {
	return '&hellip;';
}
add_filter( 'excerpt_more', 'slope_excerpt_more' );

/**
 * Custom excerpt length.
 *
 * @param  int $length Default excerpt length.
 * @return int         Custom excerpt length.
 */
function slope_excerpt_length( $length ) {
	if ( '' !== slope_get_thememod_value( 'post-excerpt-length' ) ) {
		return absint( slope_get_thememod_value( 'post-excerpt-length' ) );
	}

	return 20;
}
add_filter( 'excerpt_length', 'slope_excerpt_length', 999 );

/**
 * Like get_template_part() put lets you pass args to the template file
 * Args are available in the template as $template_args array.
 *
 * @param  string $file          Filename.
 * @param  array  $template_args Template arguments.
 * @param  array  $cache_args    Arguments for cache.
 * @return mixed
 */
function slope_get_template_part( $file, $template_args = array(), $cache_args = array() ) {
	$template_args = wp_parse_args( $template_args );
	$cache_args    = wp_parse_args( $cache_args );
	if ( $cache_args ) {
		foreach ( $template_args as $key => $value ) {
			if ( is_scalar( $value ) || is_array( $value ) ) {
				$cache_args[ $key ] = $value;
			} elseif ( is_object( $value ) && method_exists( $value, 'get_id' ) ) {
				$cache_args[ $key ] = call_user_func( 'get_id', $value );
			}
		}

		$cache = wp_cache_get( $file, serialize( $cache_args ) );
		if ( false !== $cache ) {
			if ( ! empty( $template_args['return'] ) ) {
				return $cache;
			}
			echo $cache; // WPCS: XSS ok.
			return;
		}
	}
	$file_handle = $file;
	do_action( 'start_operation', 'hm_template_part::' . $file_handle );
	if ( file_exists( get_stylesheet_directory() . '/' . $file . '.php' ) ) {
		$file = get_stylesheet_directory() . '/' . $file . '.php';
	} elseif ( file_exists( get_template_directory() . '/' . $file . '.php' ) ) {
		$file = get_template_directory() . '/' . $file . '.php';
	}
	ob_start();
	$return = require $file;
	$data   = ob_get_clean();
	do_action( 'end_operation', 'hm_template_part::' . $file_handle );
	if ( $cache_args ) {
		wp_cache_set( $file, $data, serialize( $cache_args ), 3600 );
	}
	if ( ! empty( $template_args['return'] ) ) {
		if ( false === $return ) {
			return false;
		} else {
			return $data;
		}
	}

	echo $data; // WPCS: XSS ok.
}

if ( ! function_exists( 'slope_list_authors' ) ) :
	/**
	 * Display a list of all site contributors who published at least one post.
	 *
	 * @since Slope 1.0
	 */
	function slope_list_authors() {
		$contributor_ids = get_users(
			apply_filters(
				'slope_list_authors', [
					'fields'  => 'ID',
					'orderby' => 'post_count',
					'order'   => 'DESC',
					'who'     => 'authors',
				]
			)
		);

		$authors_count = count( $contributor_ids );

		$class = '';
		if ( 1 === $authors_count ) {
			$class = 'one-column';
		} elseif ( 2 === $authors_count ) {
			$class = 'two-column';
		} elseif ( 3 === $authors_count ) {
			$class = 'three-column';
		} elseif ( 4 === $authors_count || $authors_count > 4 ) {
			$class = 'four-column';
		}

		foreach ( $contributor_ids as $contributor_id ) :
			$post_count = count_user_posts( $contributor_id );

			// Move on if user has not published a post (yet).
			if ( ! $post_count ) {
				continue;
			}
		?>

		<div class="contributor <?php echo esc_attr( $class ); ?>">
		<div class="contributor-info">
			<figure class="contributor-avatar">
				<a class="contributor-posts-link" href="<?php echo esc_url( get_author_posts_url( $contributor_id ) ); ?>">
					<?php echo get_avatar( $contributor_id, 255 ); ?>
				</a>
			</figure>
			<div class="contributor-summary">
				<h4 class="contributor-name">
					<a class="contributor-posts-link" href="<?php echo esc_url( get_author_posts_url( $contributor_id ) ); ?>">
						<?php echo get_the_author_meta( 'display_name', $contributor_id ); // WPCS: XSS ok. ?>
					</a>
				</h4>
				<div class="contributor-description">
					<?php $allowed_tags = slope_allowed_html(); ?>
					<?php echo wp_kses( get_the_author_meta( 'description', $contributor_id ), $allowed_tags ); ?>
				</div>

				<a href="<?php echo esc_url( get_author_posts_url( $contributor_id ) ); ?>" class="contributor-archive-link"><?php esc_html_e( 'See more', 'slope' ); ?></a>
			</div><!-- .contributor-summary -->
		</div><!-- .contributor-info -->
	</div><!-- .contributor -->

	<?php
	endforeach;
	}
endif;

if ( ! function_exists( 'slope_related_posts' ) ) :
	/**
	 * Get related posts for specific post.
	 * Returns 2-3 posts depending on the Single Post sidebar visibility.
	 *
	 * @param int $post_id Post ID.
	 * @return void
	 */
	function slope_related_posts( $post_id = false ) {
		if ( ! $post_id ) {
			$post_id = get_the_ID();
		}

		if ( true === slope_get_thememod_value( 'bool-hide-related-posts' ) ) {
			return;
		}

		$tags    = wp_get_post_tags( $post_id );
		$tag_ids = array();
		foreach ( $tags as $tag ) {
			$tag_ids[] = $tag->term_id;
		}

		if ( empty( $tag_ids ) ) {
			return;
		}

		$columns_count    = 2;
		$sidebar_position = slope_get_thememod_value( 'single-sidebar' );

		if ( 'none' === $sidebar_position || ! is_active_sidebar( 'sidebar-2' ) ) {
			$columns_count = 3;
		}

		$posts = new WP_Query(
			apply_filters(
				'slope_related_posts', array(
					'post_type'           => 'post',
					'tag__in'             => $tag_ids,
					'ignore_sticky_posts' => true,
					'posts_per_page'      => $columns_count,
				)
			)
		);

		if ( $posts->have_posts() ) {
			?>
			<section class="related-posts highlight-heading">
				<h2 class="related-posts__title"><?php echo esc_html_x( 'Related Posts', 'related posts title', 'slope' ); ?></h2>
				<div class="grid" data-columns>
			<?php
			while ( $posts->have_posts() ) {
				$posts->the_post();

				remove_filter( 'the_content', 'sharing_display', 19 );
				remove_filter( 'the_excerpt', 'sharing_display', 19 );

				get_template_part( 'template-parts/content' );
			}
			?>
			</div></section>
			<?php

			wp_reset_postdata();
		}
	}
endif; // End of slope_related_posts.

if ( ! function_exists( 'slope_get_category_image' ) ) :
	/**
	 * Get specific category images thumbnail.
	 *
	 * @uses WPCustomCategoryImage
	 * @see https://wordpress.org/plugins/wpcustom-category-image/
	 *
	 * @param string $term_id  Category ID.
	 * @param string $size     Thumbnail size.
	 * @return mixed|string
	 */
	function slope_get_category_image( $term_id = null, $size = 'full' ) {
		if ( ! class_exists( 'WPCustomCategoryImage' ) || ! is_category() ) {
			return false;
		}

		// Get only image url.
		$params = array(
			'term_id' => $term_id,
			'size'    => $size,
		);

		return category_image_src( $params, false ); // WPCS: XSS ok.
	}
endif; // End of slope_get_category_image.

if ( ! function_exists( 'slope_widget_screen_css' ) ) :
	/**
	 * Add custom CSS to widget screen to highlight custom widgets.
	 *
	 * @param string $hook Current screen ID.
	 * @return void
	 */
	function slope_widget_screen_css( $hook ) {
		if ( 'widgets.php' !== $hook ) {
			return;
		}

		$accent_color = slope_get_thememod_value( 'color-accent' );

		wp_add_inline_style(
			'widgets', "div[id*='_slope_'] .widget-top{
				border-left: 4px solid {$accent_color};
			}"
		);
	}
endif; // End of slope_widget_screen_css.

add_action( 'admin_enqueue_scripts', 'slope_widget_screen_css' );

if ( ! function_exists( 'slope_blog_columns_class' ) ) :
	/**
	 * Returns HTML class for blog columns/sidebar layout.
	 *
	 * @return string
	 */
	function slope_blog_columns_class() {
		$blog_columns     = slope_get_thememod_value( 'blog-column' );
		$position         = slope_get_thememod_value( 'blog-sidebar-position' );
		$sidebar_position = 'sidebar-' . $position;
		$container_class  = '';

		switch ( $blog_columns ) :
			case 2:
				$container_class = $sidebar_position;
				break;

			case 3:
				$container_class = 'column-3';
				break;

			case 4:
				$container_class = 'column-4';
				break;

			default:
				$container_class = 'column-3';
				break;
		endswitch;

		return $container_class;
	}
endif; // End of slope_blog_columns_class.
