<?php
/**
 * Custom template tags for this theme
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @package Slope
 */

if ( ! function_exists( 'slope_posted_on' ) ) :
	/**
	 * Prints HTML with meta information for the current post-date/time and author.
	 */
	function slope_posted_on() {
		$time_string = '<time class="entry-date published updated" datetime="%1$s">%2$s</time>';
		if ( get_the_time( 'U' ) !== get_the_modified_time( 'U' ) ) {
			$time_string = '<time class="entry-date published" datetime="%1$s">%2$s</time><time class="updated" datetime="%3$s">%4$s</time>';
		}

		$time_string = sprintf(
			$time_string,
			esc_attr( get_the_date( 'c' ) ),
			esc_html( get_the_date() ),
			esc_attr( get_the_modified_date( 'c' ) ),
			esc_html( get_the_modified_date() )
		);

		$posted_on = '<a href="' . esc_url( get_permalink() ) . '" rel="bookmark">' . $time_string . '</a>';

		$byline = sprintf(
			/* translators: %s: post author. */
			esc_html_x( 'by %s', 'post author', 'slope' ),
			'<span class="author vcard"><a class="url fn n" href="' . esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ) . '">' . esc_html( get_the_author() ) . '</a></span>'
		);

		echo '<span class="posted-on">' . $posted_on . '</span><span class="byline"> ' . $byline . '</span>'; // WPCS: XSS OK.

		edit_post_link(
			sprintf(
				wp_kses(
					/* translators: %s: Name of current post. Only visible to screen readers, there is a blank space before the text. */
					__( ' Edit <span class="screen-reader-text">%s</span>', 'slope' ),
					array(
						'span' => array(
							'class' => array(),
						),
					)
				),
				get_the_title()
			),
			'<span class="edit-link">',
			'</span>'
		);
	}
endif;

if ( ! function_exists( 'slope_entry_footer' ) ) :
	/**
	 * Prints HTML with meta information for the categories, tags and comments.
	 */
	function slope_entry_footer() {
		// Hide category and tag text for pages.
		if ( 'post' === get_post_type() ) {
			/* translators: used between list items, there is a space after the comma */
			$categories_list = get_the_category_list( esc_html__( ', ', 'slope' ) );
			if ( $categories_list && slope_categorized_blog() ) {
				/* translators: 1: list of categories. */
				printf( '<div class="cat-links">' . esc_html__( 'Categories: %1$s', 'slope' ) . '</div>', $categories_list ); // WPCS: XSS OK.
			}

			/* translators: used between list items, there is a space after the comma */
			$tags_list = get_the_tag_list( '', esc_html_x( ', ', 'list item separator', 'slope' ) );
			if ( $tags_list ) {
				/* translators: 1: list of tags. */
				printf( '<div class="tags-links">' . esc_html__( 'Tags: %1$s', 'slope' ) . '</div>', $tags_list ); // WPCS: XSS OK.
			}
		}

		if ( ! is_single() && ! post_password_required() && ( comments_open() || get_comments_number() ) ) {
			echo '<span class="comments-link">';
			comments_popup_link(
				sprintf(
					wp_kses(
						/* translators: %s: post title */
						__( 'Leave a Comment<span class="screen-reader-text"> on %s</span>', 'slope' ),
						array(
							'span' => array(
								'class' => array(),
							),
						)
					),
					get_the_title()
				)
			);
			echo '</span>';
		}
	}
endif;

/**
 * Returns true if a blog has more than 1 category.
 *
 * @return bool
 */
function slope_categorized_blog() {
	$all_the_cool_cats = get_transient( 'slope_categories' );
	if ( false === $all_the_cool_cats ) {
		// Create an array of all the categories that are attached to posts.
		$all_the_cool_cats = get_categories(
			array(
				'fields'     => 'ids',
				'hide_empty' => 1,
				// We only need to know if there is more than one category.
				'number'     => 2,
			)
		);

		// Count the number of categories that are attached to the posts.
		$all_the_cool_cats = count( $all_the_cool_cats );

		set_transient( 'slope_categories', $all_the_cool_cats );
	}

	if ( $all_the_cool_cats > 1 || is_preview() ) {
		// This blog has more than 1 category so slope_categorized_blog should return true.
		return true;
	} else {
		// This blog has only 1 category so slope_categorized_blog should return false.
		return false;
	}
}

/**
 * Flush out the transients used in slope_categorized_blog.
 */
function slope_category_transient_flusher() {
	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
		return;
	}
	// Like, beat it. Dig?
	delete_transient( 'slope_categories' );
}
add_action( 'edit_category', 'slope_category_transient_flusher' );
add_action( 'save_post', 'slope_category_transient_flusher' );

/**
 * Get a sanitized value for a Theme Mod setting.
 *
 * @param var $setting_id Setting ID.
 *
 * @return mixed
 */
function slope_get_thememod_value( $setting_id ) {
	global $slope_settings;

	return $slope_settings->get_value( $setting_id );
}

/**
 * Provides a fallback images for single post, pages, and archive pages.
 *
 * @return string|void
 */
function slope_blog_fallback_image() {
	$cover_image = slope_get_thememod_value( 'fallback-blog-image' );

	if ( '' !== $cover_image ) {
		return $cover_image;
	}

	return false;
}

/**
 * Returns feature image URL for post/pages.
 *
 * @uses slope_is_woocommerce_page()
 * @uses slope_blog_fallback_image() to set fallback featured image.
 *
 * @param var $post_id Post ID.
 * @return string|void
 */
function slope_featured_image_cover( $post_id = '' ) {
	if ( '' === $post_id ) {
		$post_id = get_the_ID();
	}

	$featured_image = get_the_post_thumbnail_url( get_the_ID(), 'full' );
	$fallback_image = slope_blog_fallback_image();

	if ( slope_is_woocommerce_page() ) {
		$fallback_image = slope_get_thememod_value( 'fallback-shop-image' );
	}

	// Return early, if featured image exists.
	if ( has_post_thumbnail( $post_id ) ) {
		return $featured_image;
	}

	// Fallback images for post cover if featured image doesn't exists.
	if ( '' !== $fallback_image ) {
		return $fallback_image;
	}

	// Retrun false, if still here.
	return false;
}
