<?php
/**
 * Page: Posts widget.
 *
 * @package Slope
 */

if ( ! class_exists( 'Slope_Widget_Category_Fold' ) ) :
	/**
	 * Display recent posts from specific category.
	 *
	 * @package Slope
	 */
	class Slope_Widget_Category_Fold extends Slope_Widget {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->widget_id          = 'slope_category_fold';
			$this->widget_description = esc_html__( 'Displays posts from a specific category in a different style.', 'slope' );
			$this->widget_name        = esc_html__( 'Slope Section: Category Fold', 'slope' );
			$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
			$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
			$this->settings           = array(
				'title' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Title:', 'slope' ),
				),
				'subtitle' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Subtitle:', 'slope' ),
				),
				'category' => array(
					'type'        => 'category',
					'std'         => '0',
					'label'       => esc_html__( 'Category:', 'slope' ),
				),
				'posts_count' => array(
					'type'  => 'number',
					'std'   => '3',
					'step'  => '1',
					'min'   => '1',
					'max'   => '1000',
					'label' => esc_html__( 'Number of posts:', 'slope' ),
				),
				'image' => array(
					'type'  => 'image',
					'std'   => null,
					'label' => esc_html__( 'Fold Image:', 'slope' ),
				),
				'background_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#fff',
					'label' => esc_html__( 'Background Color:', 'slope' ),
				),
				'text_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#191919',
					'label' => esc_html__( 'Text Color:', 'slope' ),
				),
				'section_padding' => array(
					'type'        => 'select',
					'std'         => 'standard',
					'label'       => esc_html__( 'Section padding:', 'slope' ),
					'options'     => array(
						'100px' => esc_html__( 'Standard', 'slope' ),
						'50px'  => esc_html__( 'Half', 'slope' ),
						'0'     => esc_html__( 'None', 'slope' ),
					),
				),
				'padding_description' => array(
					'type' => 'description',
					'std'  => esc_html__( 'Determines top and bottom padding for the section.', 'slope' ),
				),
			);

			parent::__construct();
		}

		/**
		 * Output the widget content on the page.
		 *
		 * @since 1.0.0
		 *
		 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current widget instance.
		 */
		public function widget( $args, $instance ) {
			// Check widget areas context.
			if ( ! is_singular( 'page' ) ) {
				echo $this->widget_areas_notice(); // WPCS: XSS ok.

				return false;
			}

			$title            = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );
			$subtitle         = isset( $instance['subtitle'] ) ? esc_attr( $instance['subtitle'] ) : false;
			$category         = isset( $instance['category'] ) ? esc_attr( $instance['category'] ) : '0';
			$posts_count      = isset( $instance['posts_count'] ) ? esc_attr( $instance['posts_count'] ) : '3';
			$image            = isset( $instance['image'] ) ? esc_url( $instance['image'] ) : null;
			$background_color = maybe_hash_hex_color( $instance['background_color'] );
			$text_color       = maybe_hash_hex_color( $instance['text_color'] );
			$section_padding  = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '100px';
			$category_link    = get_category_link( $category );

			/**
			* Filters the arguments for the Posts widget.
			*
			* @see WP_Query::get_posts()
			*
			* @param array $args An array of arguments used to retrieve the recent posts.
			*/
			$r = new WP_Query( apply_filters( 'slope_widget_category_fold_args', array(
				'posts_per_page'      => $posts_count,
				'no_found_rows'       => true,
				'cat'                 => $category,
				'post_status'         => 'publish',
				'ignore_sticky_posts' => true,
			) ) );

			if ( $r->have_posts() ) :
			echo $args['before_widget']; // WPCS: XSS ok.
			?>

			<style type="text/css">
				#<?php echo esc_html( $this->id ); ?> {
					background-color: <?php echo esc_html( $background_color ); ?>;
					color: <?php echo esc_html( $text_color ); ?>;
				}
				#<?php echo esc_html( $this->id ); ?> .fold-content {
					padding-top: <?php echo esc_html( $section_padding ); ?>;
					padding-bottom: <?php echo esc_html( $section_padding ); ?>;
				}
				#<?php echo esc_html( $this->id ); ?> .grid__tile {
					border-color: <?php echo esc_html( $background_color ); ?>;
				}

				#<?php echo esc_html( $this->id ); ?> .fold-cover {
					background-image: url(<?php echo esc_url( $image ); ?>);
				}

			</style>

			<section class="container">
				<div class="fold-content">
					<div class="home-widget-section-title">
					<?php
						if ( $title ) {
						echo $args['before_title']; // WPCS: XSS ok.
						echo $title;  // WPCS: XSS ok.

						if ( $category_link ) :
						?>
						<a href="<?php echo esc_url( $category_link ); ?>" class="category-link"><?php esc_html_e( 'View more posts &rarr;', 'slope' ); ?></a>
						<?php
						endif;

						echo $args['after_title'];  // WPCS: XSS ok.
					}
					?>

					<?php if ( $subtitle ) : ?>
					<p class="home-widget-subtitle"><?php echo esc_attr( $subtitle ); ?></p>
					<?php endif; ?>
				</div>

				<?php $count = 1; ?>
				<div class="category-fold-posts">
					<?php while ( $r->have_posts() ) : ?>
						<?php $r->the_post(); ?>

						<article id="post-<?php the_ID(); ?>" <?php post_class(); ?>>
							<?php if ( 1 !== $count ) : ?>
							<time class="entry-date" datetime="<?php the_time( 'c' ); ?>"><?php the_time( get_option( 'date_format' ) ); ?></time>
							<?php endif; ?>

							<h3 class="entry-title">
								<a href="<?php the_permalink(); ?>"><?php the_title(); ?></a>
							</h3>

							<?php if ( 1 === $count ) : ?>
							<div class="entry-excerpt">
								<?php the_excerpt(); ?>
							</div>

							<a href="<?php the_permalink(); ?>" class="button"><?php esc_html_e( 'Read More', 'slope' ); ?></a>
							<?php endif; ?>
						</article>
						<?php $count++; ?>
					<?php endwhile; ?>
				</div>
				</div>
				<div class="fold-cover"></div>
			</section>

			<?php
			wp_reset_postdata();

			echo $args['after_widget']; // WPCS: XSS ok.

			endif; // End $r->have_posts() check.
		}

		/**
		 * Registers the widget with the WordPress Widget API.
		 *
		 * @return mixed
		 */
		public static function register() {
			register_widget( __CLASS__ );
		}
	}
endif;

add_action( 'widgets_init', array( 'Slope_Widget_Category_Fold', 'register' ) );
