<?php
/**
 * Page: Callouts widget.
 *
 * @package Slope
 */

/**
 * Feature collout widget for widgetized pages.
 *
 * @package Slope
 */
class Slope_Widget_Feature_Callout extends Slope_Widget {
	/**
	 * Constructor
	 */
	public function __construct() {
		$this->widget_id          = 'slope_feature_callout';
		$this->widget_description = esc_html__( 'Displays a feature callout.', 'slope' );
		$this->widget_name        = esc_html__( 'Slope Section: Feature Callout', 'slope' );
		$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
		$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
		$this->settings           = array(
			'title' => array(
				'type'  => 'text',
				'std'   => '',
				'label' => __( 'Title:', 'slope' ),
			),
			'content' => array(
				'type'  => 'textarea',
				'std'   => '',
				'label' => __( 'Content:', 'slope' ),
				'rows'  => 5,
			),
			'text_align' => array(
				'type'  => 'select',
				'std'   => 'left',
				'label' => __( 'Text Align:', 'slope' ),
				'options' => array(
					'left' => __( 'Left', 'slope' ),
					'right' => __( 'Right', 'slope' ),
					'center' => __( 'Center (cover only)', 'slope' ),
				),
			),
			'vertical_align' => array(
				'type'  => 'select',
				'std'   => 'middle',
				'label' => __( 'Vertical Alignment:', 'slope' ),
				'options' => array(
					'top' => __( 'Top', 'slope' ),
					'middle' => __( 'Middle', 'slope' ),
					'bottom' => __( 'Bottom', 'slope' ),
				),
			),
			'image' => array(
				'type'  => 'image',
				'std'   => null,
				'label' => esc_html__( 'Image:', 'slope' ),
			),
			'background' => array(
				'type'  => 'select',
				'std'   => 'pull',
				'label' => __( 'Image Style:', 'slope' ),
				'options' => array(
					'cover' => __( 'Cover', 'slope' ),
					'pull'  => __( 'Pull Out', 'slope' ),
				),
			),
			'cover_overlay' => array(
				'type' => 'checkbox',
				'std'  => 1,
				'label' => __( 'Use dark transparent overlay (cover only)', 'slope' ),
			),
			'text_color' => array(
				'type'  => 'colorpicker',
				'std'   => '#2b2828',
				'label' => __( 'Text Color:', 'slope' ),
			),
			'background_color' => array(
				'type'  => 'colorpicker',
				'std'   => '#ffffff',
				'label' => __( 'Background Color:', 'slope' ),
			),
			'section_padding' => array(
				'type'        => 'select',
				'std'         => 'standard',
				'label'       => esc_html__( 'Section padding:', 'slope' ),
				'options'     => array(
					'100px' => esc_html__( 'Standard', 'slope' ),
					'50px'  => esc_html__( 'Half', 'slope' ),
					'0'     => esc_html__( 'None', 'slope' ),
				),
			),
			'padding_description' => array(
				'type' => 'description',
				'std'  => esc_html__( 'Determines top and bottom padding for the section.', 'slope' ),
			),
		);

		parent::__construct();

		add_filter( 'atik_feature_callout_description', 'wptexturize' );
		add_filter( 'atik_feature_callout_description', 'convert_smilies' );
		add_filter( 'atik_feature_callout_description', 'convert_chars' );
		add_filter( 'atik_feature_callout_description', 'wpautop' );
		add_filter( 'atik_feature_callout_description', 'shortcode_unautop' );
		add_filter( 'atik_feature_callout_description', 'prepend_attachment' );
		add_filter( 'atik_feature_callout_description', 'do_shortcode' );
	}

	/**
	 * Output the widget content on the page.
	 *
	 * @since 1.0.0
	 *
	 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
	 * @param array $instance Settings for the current widget instance.
	 */
	public function widget( $args, $instance ) {
		// Check widget areas context.
		if ( ! is_singular( 'page' ) ) {
			echo $this->widget_areas_notice(); // WPCS: XSS ok.

			return false;
		}

		$text_align       = isset( $instance['text_align'] ) ? esc_attr( $instance['text_align'] ) : 'left';
		$vertical_align   = isset( $instance['vertical_align'] ) ? esc_attr( $instance['vertical_align'] ) : 'middle';
		$title            = isset( $instance['title'] ) ? esc_attr( $instance['title'] ) : '';
		$background       = isset( $instance['background'] ) ? esc_attr( $instance['background'] ) : 'cover';
		$background_color = isset( $instance['background_color'] ) ? esc_attr( $instance['background_color'] ) : '#ffffff';
		$image            = isset( $instance['image'] ) ? esc_url( $instance['image'] ) : null;
		$overlay          = isset( $instance['cover_overlay'] ) && 1 === absint( $instance['cover_overlay'] ) ? 'has-overlay' : 'no-overlay';
		$section_padding  = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '100px';
		$content          = $this->callout_content( $instance );

		ob_start(); ?>

		<?php echo $args['before_widget']; // WPCS: XSS ok. ?>

		<style type="text/css">
			#<?php echo esc_html( $this->id ); ?> .feature-callout {
				padding: <?php echo esc_html( $section_padding ); ?> 0;
			}
		</style>

		<div class="feature-callout text-<?php echo esc_attr( $text_align ); ?> image-<?php echo esc_attr( $background ); ?>"  style="background-color: <?php echo esc_attr( $background_color ); ?>;">
			<?php if ( 'pull' === $background ) : ?>
				<div class="container">
					<div class="grid grid--no-gutter valign-<?php echo esc_attr( $vertical_align ); ?>">
						<div class="grid__col grid__col--1-of-2 text-container <?php echo ( 'right' === $text_align ) ? 'grid__col--push-1-of-2' : ''; ?>"><?php echo $content; // WPCS: XSS ok. ?></div>

						<?php if ( '' !== $image ) : ?>
						<div class="grid__col grid__col--1-of-2 image-container <?php echo ( 'right' === $text_align ) ? 'grid__col--pull-2-of-2' : ''; ?>">
							<img src="<?php echo esc_url( $image ); ?>" alt="<?php echo esc_attr( $title ); ?>">
						</div>
						<?php endif; ?>
					</div>
				</div>
			<?php else : ?>
				<div class="feature-callout__cover <?php echo esc_attr( $overlay ); ?>" style="background-image:url(<?php echo esc_url( $image ); ?>); ?>;">
					<div class="container">
						<div class="grid">
							<?php
							$align_class = 'grid__col--1-of-2';
							if ( 'left' === $text_align ) {
								$align_class = 'grid__col--1-of-2';
							} elseif ( 'right' === $text_align ) {
								$align_class = 'grid__col--1-of-2 grid__col--push-1-of-2';
							} elseif ( 'center' === $text_align ) {
								$align_class = 'grid__col--3-of-5 grid__col--centered';
							}
							?>
							<div class="grid__col <?php echo esc_attr( $align_class ); ?>">
								<?php echo $content; // WPCS: XSS ok. ?>
							</div>
						</div>
					</div>
				</div>
			<?php endif; ?>
		</div>

		<?php
		echo $args['after_widget']; // WPCS: XSS ok.
	}

	/**
	 * Callout content in a nice format.
	 *
	 * @param array $instance Settings for the current widget instance.
	 * @return mixed Callout content.
	 */
	private function callout_content( $instance ) {
		$text_color = isset( $instance['text_color'] ) ? esc_attr( $instance['text_color'] ) : '#2b2828';
		$title      = isset( $instance['title'] ) ? esc_attr( $instance['title'] ) : '';
		$content    = isset( $instance['content'] ) ? $instance['content'] : '';

		$output  = '<div class="feature-callout__content" style="color:' . $text_color . '">';
		$output .= '<h2 class="feature-callout__title" style="color:' . $text_color . '">' . $title . '</h2>';
		$output .= wpautop( $content );
		$output .= '</div>';

		$output  = apply_filters( 'atik_feature_callout_description', $output );

		return $output;
	}

	/**
	 * Registers the widget with the WordPress Widget API.
	 *
	 * @return mixed
	 */
	public static function register() {
		register_widget( __CLASS__ );
	}
}

add_action( 'widgets_init', array( 'Slope_Widget_Feature_Callout', 'register' ) );
