<?php
/**
 * Page: Features Widget
 *
 * @package Slope
 */

if ( ! class_exists( 'Slope_Widget_Features' ) ) :
	/**
	 * Display static content from an specific page.
	 *
	 * @package Slope
	 */
	class Slope_Widget_Features extends Slope_Widget {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->widget_id          = 'slope_features';
			$this->widget_description = esc_html__( 'Displays features about your site.', 'slope' );
			$this->widget_name        = esc_html__( 'Slope Section: Features', 'slope' );
			$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
			$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
			$this->settings           = array(
				'title' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Title:', 'slope' ),
				),
				'subtitle' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Subtitle:', 'slope' ),
				),
				'is_centered' => array(
					'type' => 'checkbox',
					'std'  => 0,
					'label' => __( 'Center Content', 'slope' ),
				),
				'section_padding' => array(
					'type'        => 'select',
					'std'         => 'standard',
					'label'       => esc_html__( 'Section padding:', 'slope' ),
					'options'     => array(
						'100px' => esc_html__( 'Standard', 'slope' ),
						'50px'  => esc_html__( 'Half', 'slope' ),
						'0'     => esc_html__( 'None', 'slope' ),
					),
				),
				'padding_description' => array(
					'type' => 'description',
					'std'  => esc_html__( 'Determines top and bottom padding for the section.', 'slope' ),
				),
				'features' => array(
					'type' => 'features', // Custom type. See Below.
					'std'  => array(),
					'label' => '',
				),
			);

			parent::__construct();

			add_action( 'admin_enqueue_scripts', array( $this, 'admin_enqueue_scripts' ) );
			add_action( 'slope_widget_type_features', array( $this, 'output' ), 10, 4 );

			add_action( 'admin_footer', array( $this, 'load_js_template' ) );
			add_action( 'customize_controls_print_scripts', array( $this, 'load_js_template' ) );
		}

		/**
		 * Widget Features Admin Scripts.
		 *
		 * @param string $pagenow Current page ID.
		 * @since unknown
		 */
		public function admin_enqueue_scripts( $pagenow ) {
			// Only in widgets.php and customizer.
			global $pagenow;

			if ( ! in_array( $pagenow, array( 'widgets.php', 'customize.php' ), true ) ) {
				return;
			}

			wp_enqueue_script( 'slope-admin-widget-features', get_template_directory_uri() . '/js/admin/widget-features.js', array( 'wp-util', 'underscore', 'backbone', 'jquery', 'jquery-ui-sortable' ) );
		}

		/**
		 * Output the widget content on the page.
		 *
		 * @since 1.0.0
		 *
		 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current widget instance.
		 */
		public function widget( $args, $instance ) {
			// Check widget areas context.
			if ( ! is_singular( 'page' ) ) {
				echo $this->widget_areas_notice(); // WPCS: XSS ok.

				return false;
			}

			// Get features.
			$features = isset( $instance['features'] ) ? $instance['features'] : array();

			// Do not display widget if no features defined.
			if ( empty( $features ) ) {
				return;
			}

			$title           = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );
			$subtitle        = isset( $instance['subtitle'] ) ? esc_attr( $instance['subtitle'] ) : false;
			$is_centered     = isset( $instance['is_centered'] ) ? esc_attr( $instance['is_centered'] ) : false;
			$section_padding = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '100px';

			echo $args['before_widget']; // WPCS: XSS ok.
			?>
			<style type="text/css">
				#<?php echo esc_html( $this->id ); ?> {
					padding: <?php echo esc_html( $section_padding ); ?> 0;
				}
			</style>

			<section class="container<?php if ( $is_centered ) echo ' home-widget--centered'; ?>">
				<div class="home-widget-section-title">
					<?php if ( $title ) echo $args['before_title'] . $title . $args['after_title']; // WPCS: XSS ok, sanitization ok. ?>

					<?php if ( $subtitle ) : ?>
					<p class="home-widget-subtitle"><?php echo esc_attr( $subtitle ); ?></p>
					<?php endif; ?>
				</div>

				<div class="grid features">
				<?php
				foreach ( $features as $feature ) {
					if ( is_object( $feature ) ) {
						$feature = json_decode( wp_json_encode( $feature ), true );
					}

					$title       = isset( $feature['title'] ) ? $feature['title'] : null;
					$description = isset( $feature['description'] ) ? apply_filters( 'slope_feature_description', $feature['description'] ) : null;
					$media       = esc_url( $feature['media'] );
					$button_text = isset( $feature['button_text'] ) ? $feature['button_text'] : null;
					$button_url  = esc_url( $feature['button_url'] );

					$feature = compact( 'title', 'description', 'media', 'button_text', 'button_url' );
					slope_get_template_part( 'template-parts/content-home-feature', array( 'feature' => $feature ) );
				}
				?>
				</div>
			</section>

			<?php
			echo $args['after_widget']; // WPCS: XSS ok.
		}

		/**
		 * Widget Settings HTML Output.
		 *
		 * @since unknown
		 * @param object $widget   Widget.
		 * @param string $key      Field key, value is "features".
		 * @param array  $setting  Setting.
		 * @param array  $instance Instance.
		 * @return void
		 */
		public function output( $widget, $key, $setting, $instance ) {
			$features = isset( $instance[ $key ] ) ? $instance[ $key ] : $setting['std'];
			?>

	<div id="features-<?php echo esc_attr( $widget->id ); ?>" class="features-wrap">

		<p><a href="#" class="button-add-feature button button-secondary"><?php esc_html_e( 'Add Feature', 'slope' ); ?></a></p>

		<div class="features"></div>

	</div>

	<script>
		jQuery(document).ready(function($) {
			var feature_template = wp.template( 'feature' );
			<?php
				foreach ( $features as $order => $feature ) :
				$feature['widget_num'] = $widget->number;
				$feature['order'] = $order;
				?>
				$( '#features-<?php echo esc_attr( $widget->id ); ?> .features' ).append( feature_template( <?php echo wp_json_encode( $feature ); ?> ) ).sortable();
			<?php endforeach; ?>
		});
	</script>

		<?php
		}

		/**
		 * JS Template & Style.
		 */
		public function load_js_template() {
			// Only in widgets.php and customizer. And this need to be loaded only once.
			global $pagenow, $_slope_widget_home_features_tmpl_loaded;

			if ( ! in_array( $pagenow, array( 'widgets.php', 'customize.php' ), true ) && isset( $_slope_widget_home_features_tmpl_loaded ) && $_slope_widget_home_features_tmpl_loaded ) {
				return;
			}

			$_slope_widget_home_features_tmpl_loaded = true;

			?>

	<script id="tmpl-feature" type="text/template">
		<div class="feature">
			<a href="#" class="button-remove-feature">&nbsp;</a>

			<p>
				<label><?php esc_html_e( 'Title:', 'slope' ); ?></label>
				<input name="widget-slope_features[{{data.widget_num}}][features][{{data.order}}][title]" type="text" value="{{data.title}}" class="widefat" />
			</p>

			<p>
				<label><?php esc_html_e( 'Media:', 'slope' ); ?></label>
				<input type="text" class="widefat" name="widget-slope_features[{{data.widget_num}}][features][{{data.order}}][media]" value="{{data.media}}" placeholder="http://" />
			</p>

			<p>
				<label><?php esc_html_e( 'Description:', 'slope' ); ?></label>
				<textarea name="widget-slope_features[{{data.widget_num}}][features][{{data.order}}][description]" rows="3" class="widefat">{{{data.description}}}</textarea>
			</p>

			<p>
				<label><?php esc_html_e( 'Button Text:', 'slope' ); ?></label>
				<input name="widget-slope_features[{{data.widget_num}}][features][{{data.order}}][button_text]" type="text" value="{{data.button_text}}" class="widefat" />
			</p>

			<p>
				<label><?php esc_html_e( 'Button URL:', 'slope' ); ?></label>
				<input type="text" class="widefat" name="widget-slope_features[{{data.widget_num}}][features][{{data.order}}][button_url]" value="{{data.button_url}}" placeholder="http://" />
			</p>
		</div>
	</script>

	<style>
		.feature {
			border: 1px solid #ddd;
			margin-bottom: 1em;
			padding: 0.5em 1em;
			background: #fff;
			cursor: move;
			position: relative;
		}

		.button-remove-feature {
			position: absolute;
			top: 5px;
			right: 5px;
			text-decoration: none;
		}

		.button-remove-feature:before {
			background: 0 0;
			color: #BBB;
			content: '\f153';
			display: block!important;
			font: 400 13px/1 dashicons;
			speak: none;
			height: 20px;
			margin: 2px 0;
			text-align: center;
			width: 20px;
			-webkit-font-smoothing: antialiased!important;
		}
	</style>

			<?php
		}

		/**
		 * Registers the widget with the WordPress Widget API.
		 *
		 * @return mixed
		 */
		public static function register() {
			register_widget( __CLASS__ );
		}
	}
endif;

add_action( 'widgets_init', array( 'Slope_Widget_Features', 'register' ) );
