<?php
/**
 * Page: Post slider Widget
 *
 * @package Slope
 */

if ( ! class_exists( 'Slope_Widget_Post_Slider' ) ) :
	/**
	 * Display slider from a set of posts.
	 *
	 * @package Slope
	 */
	class Slope_Widget_Post_Slider extends Slope_Widget {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->widget_id          = 'slope_post_slider';
			$this->widget_description = esc_html__( 'Displays slider from specific set of posts.', 'slope' );
			$this->widget_name        = esc_html__( 'Slope Section: Post Slider', 'slope' );
			$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
			$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
			$this->settings           = array(
				'slider-category' => array(
					'type'  => 'category',
					'std'   => '0',
					'label' => esc_html__( 'Category:', 'slope' ),
				),
				'slider-tags' => array(
					'type'  => 'tags',
					'std'   => '-1',
					'label' => esc_html__( 'Tags:', 'slope' ),
				),
				'animation' => array(
					'type'        => 'select',
					'std'         => 'slide',
					'label'       => esc_html__( 'Slider animation:', 'slope' ),
					'options'     => array(
						'fade'  => esc_html__( 'Fade', 'slope' ),
						'slide' => esc_html__( 'Slide', 'slope' ),
					),
				),
				'posts_count' => array(
					'type'  => 'number',
					'std'   => '4',
					'step'  => '1',
					'min'   => '1',
					'max'   => '1000',
					'label' => esc_html__( 'Number of posts:', 'slope' ),
				),
				'slider-arrows' => array(
					'type'  => 'checkbox',
					'std'   => true,
					'label' => esc_html__( 'Show navigation arrows?', 'slope' ),
				),
				'background_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#fff',
					'label' => esc_html__( 'Background Color:', 'slope' ),
				),
				'section_padding' => array(
					'type'        => 'select',
					'std'         => 'standard',
					'label'       => esc_html__( 'Section padding:', 'slope' ),
					'options'     => array(
						'100px' => esc_html__( 'Standard', 'slope' ),
						'50px'  => esc_html__( 'Half', 'slope' ),
						'0'     => esc_html__( 'None', 'slope' ),
					),
				),
				'padding_description' => array(
					'type' => 'description',
					'std'  => esc_html__( 'Determines top and bottom padding for the section.', 'slope' ),
				),
			);

			parent::__construct();

			if ( is_active_widget( false, false, $this->id_base ) || is_customize_preview() ) {
				add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
			}
		}

		/**
		 * Enqueue scripts.
		 */
		public function enqueue_scripts() {
			wp_enqueue_script( 'flexslider' );
			wp_enqueue_style( 'dashicons' );
		}

		/**
		 * Output the widget content on the page.
		 *
		 * @since 1.0.0
		 *
		 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current widget instance.
		 */
		public function widget( $args, $instance ) {
			// Check widget areas context.
			if ( ! is_singular( 'page' ) ) {
				echo $this->widget_areas_notice(); // WPCS: XSS ok.

				return false;
			}

			$category         = isset( $instance['slider-category'] ) ? esc_attr( $instance['slider-category'] ) : '0';
			$tags             = isset( $instance['slider-tags'] ) ? esc_attr( $instance['slider-tags'] ) : '-1';
			$animation        = isset( $instance['animation'] ) ? esc_attr( $instance['animation'] ) : 'slide';
			$slider_arrows    = isset( $instance['slider-arrows'] ) ? esc_attr( $instance['slider-arrows'] ) : true;
			$posts_count      = isset( $instance['posts_count'] ) ? esc_attr( $instance['posts_count'] ) : '4';
			$section_padding  = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '50px';
			$background_color = maybe_hash_hex_color( $instance['background_color'] );

			/**
			* Filters the arguments for the Post Slider widget.
			*
			* @see WP_Query::get_posts()
			*
			* @param array $args An array of arguments used to retrieve the recent posts.
			*/
			$query_args = array(
				'posts_per_page'      => $posts_count,
				'no_found_rows'       => true,
				'post_status'         => 'publish',
				'ignore_sticky_posts' => true,
				'meta_query'          => array(
					array(
						'key' => '_thumbnail_id',
					),
				),
			);

			if ( 0 < absint( $tags ) && 0 < absint( $category ) ) {
				$query_args['tax_query']['relation'] = 'OR';
			}

			if ( (int) $category > 0 ) {
				$query_args['tax_query'][] = array(
					'taxonomy' => 'category',
					'field'    => 'ID',
					'terms'    => $category,
				);
			}
			if ( (int) $tags > 0 ) {
				$query_args['tax_query'][] = array(
					'taxonomy' => 'post_tag',
					'field'    => 'ID',
					'terms'    => $tags,
				);
			}

			$r = new WP_Query( apply_filters( 'slope_widget_post_slider_args', $query_args ) );

			if ( ! $r->have_posts() ) return;

			echo $args['before_widget']; // WPCS: XSS ok.
			?>
			<style type="text/css">
				#<?php echo esc_html( $this->id ); ?> {
					margin: <?php echo esc_html( $section_padding ); ?> 0;
					background-color: <?php echo esc_html( $background_color ); ?>;
				}
			</style>
			<script>
				jQuery( document ).ready( function( $ ) {
					$( '#<?php echo esc_html( $this->id ); ?>' ).flexslider({
						animation   : '<?php echo esc_js( $animation ); ?>',
						smoothHeight: true,
						controlNav  : false,
						directionNav: '<?php echo esc_js( (bool) $slider_arrows ); ?>',
						prevText    : '<span class="background-secondary dashicons dashicons-arrow-left-alt2"></span>',
						nextText    : '<span class="background-secondary dashicons dashicons-arrow-right-alt2"></span>'
					});
				});
			</script>

			<div class="flexslider"><ul class="slides">

			<?php
				while ( $r->have_posts() ) :
				$r->the_post();

				$slider_img = get_the_post_thumbnail_url( get_the_ID(), 'full' );
			?>

			<li>
				<div class="slope-slider-wrapper">
					<div class="slope-slider-image" style='background-image:url(<?php echo esc_url( $slider_img ); ?>)'></div>

					<div class="slope-slider-content-container">
						<div class="slope-slider-content">
							<div class="background-secondary">
								<div class="entry-meta">
									<?php slope_posted_on(); ?>
								</div>

								<h2 class="slope-slider-title">
									<a href="<?php the_permalink(); ?>" rel="bookmark"><?php the_title(); ?></a>
								</h2>

								<div class="slope-slider-excerpt">
									<?php the_excerpt(); ?>
								</div>
							</div>
							<a href="<?php the_permalink(); ?>" class="button"><?php esc_html_e( 'Read More', 'slope' ); ?></a>
						</div>

					</div>
				</div>
			</li>

			<?php

			endwhile;

			echo '</ul></div>';

			echo $args['after_widget']; // WPCS: XSS ok.
		}

		/**
		 * Registers the widget with the WordPress Widget API.
		 *
		 * @return mixed
		 */
		public static function register() {
			register_widget( __CLASS__ );
		}
	}
endif;

add_action( 'widgets_init', array( 'Slope_Widget_Post_Slider', 'register' ) );
