<?php
/**
 * Page: Posts widget.
 *
 * @package Slope
 */

if ( ! class_exists( 'Slope_Widget_Posts' ) ) :
	/**
	 * Display recent posts from specific category.
	 *
	 * @package Slope
	 */
	class Slope_Widget_Posts extends Slope_Widget {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->widget_id          = 'slope_posts';
			$this->widget_description = esc_html__( 'Displays posts from a specific category.', 'slope' );
			$this->widget_name        = esc_html__( 'Slope Section: Posts', 'slope' );
			$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
			$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
			$this->settings           = array(
				'title' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Title:', 'slope' ),
				),
				'subtitle' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Subtitle:', 'slope' ),
				),
				'category' => array(
					'type'        => 'category',
					'std'         => '0',
					'label'       => esc_html__( 'Category:', 'slope' ),
				),
				'orderby' => array(
					'label'   => esc_html__( 'Order by:', 'slope' ),
					'type'    => 'select',
					'std'     => 'date',
					'options' => array(
						'date'     => esc_html__( 'Date', 'slope' ),
						'ID'       => esc_html__( 'ID', 'slope' ),
						'author'   => esc_html__( 'Author', 'slope' ),
						'title'    => esc_html__( 'Title', 'slope' ),
						'modified' => esc_html__( 'Last Modified', 'slope' ),
						'rand'     => esc_html__( 'Random', 'slope' ),
					),
				),
				'post_columns' => array(
					'label'   => esc_html__( 'Post Columns:', 'slope' ),
					'type'    => 'select',
					'std'     => '3',
					'options' => array(
						'1' => '1',
						'2' => '2',
						'3' => '3',
					),
				),
				'posts_count' => array(
					'type'  => 'number',
					'std'   => '3',
					'step'  => '1',
					'min'   => '1',
					'max'   => '1000',
					'label' => esc_html__( 'Number of posts:', 'slope' ),
				),
				'layout' => array(
					'label'   => esc_html__( 'Layout:', 'slope' ),
					'type'    => 'select',
					'std'     => 'grid',
					'options' => array(
						'tile' => esc_html__( 'Tile', 'slope' ),
						'grid' => esc_html__( 'Grid', 'slope' ),
					),
				),
				'excerpt_length' => array(
					'type'  => 'number',
					'std'   => '25',
					'step'  => '5',
					'min'   => '5',
					'max'   => '200',
					'label' => esc_html__( 'Post Excerpt Length:', 'slope' ),
				),
				'hide_excerpt' => array(
					'type' => 'checkbox',
					'std'  => 0,
					'label' => __( 'Hide Post Excerpt', 'slope' ),
				),
				'background_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#fff',
					'label' => esc_html__( 'Background Color:', 'slope' ),
				),
				'text_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#191919',
					'label' => esc_html__( 'Text Color:', 'slope' ),
				),
				'section_padding' => array(
					'type'        => 'select',
					'std'         => 'standard',
					'label'       => esc_html__( 'Section padding:', 'slope' ),
					'options'     => array(
						'100px' => esc_html__( 'Standard', 'slope' ),
						'50px'  => esc_html__( 'Half', 'slope' ),
						'0'     => esc_html__( 'None', 'slope' ),
					),
				),
				'padding_description' => array(
					'type' => 'description',
					'std'  => esc_html__( 'Determines top and bottom padding for the section.', 'slope' ),
				),
			);

			parent::__construct();
		}

		/**
		 * Output the widget content on the page.
		 *
		 * @since 1.0.0
		 *
		 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current widget instance.
		 */
		public function widget( $args, $instance ) {
			// Check widget areas context.
			if ( ! is_singular( 'page' ) ) {
				echo $this->widget_areas_notice(); // WPCS: XSS ok.

				return false;
			}

			$title            = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );
			$subtitle         = isset( $instance['subtitle'] ) ? esc_attr( $instance['subtitle'] ) : false;
			$category         = isset( $instance['category'] ) ? esc_attr( $instance['category'] ) : '0';
			$orderby          = isset( $instance['orderby'] ) ? esc_attr( $instance['orderby'] ) : 'date';
			$post_columns     = absint( $instance['post_columns'] );
			$posts_count      = isset( $instance['posts_count'] ) ? esc_attr( $instance['posts_count'] ) : '3';
			$layout           = isset( $instance['layout'] ) ? esc_attr( $instance['layout'] ) : 'grid';
			$excerpt_length   = isset( $instance['excerpt_length'] ) ? esc_attr( $instance['excerpt_length'] ) : '25';
			$hide_excerpt     = isset( $instance['hide_excerpt'] ) ? esc_attr( $instance['hide_excerpt'] ) : false;
			$background_color = maybe_hash_hex_color( $instance['background_color'] );
			$text_color       = maybe_hash_hex_color( $instance['text_color'] );
			$section_padding  = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '100px';
			$category_link    = get_category_link( $category );

			/**
			* Filters the arguments for the Posts widget.
			*
			* @see WP_Query::get_posts()
			*
			* @param array $args An array of arguments used to retrieve the recent posts.
			*/
			$r = new WP_Query( apply_filters( 'slope_widget_posts_args', array(
				'posts_per_page'      => $posts_count,
				'no_found_rows'       => true,
				'orderby'             => $orderby,
				'cat'                 => $category,
				'post_status'         => 'publish',
				'ignore_sticky_posts' => true,
			) ) );
			add_filter( 'excerpt_length', function( $arg ) use ( $excerpt_length ) {
				return $excerpt_length;
			}, 999 );

			if ( $r->have_posts() ) :
			echo $args['before_widget']; // WPCS: XSS ok.
			?>

			<style type="text/css">
				#<?php echo esc_html( $this->id ); ?> {
					background-color: <?php echo esc_html( $background_color ); ?>;
					color: <?php echo esc_html( $text_color ); ?>;
					padding: <?php echo esc_html( $section_padding ); ?> 0;
				}
				<?php if ( $hide_excerpt ) : ?>
				#<?php echo esc_html( $this->id ); ?> .entry-content {
					display: none;
				}
				<?php endif; ?>
				#<?php echo esc_html( $this->id ); ?> .grid__tile {
					border-color: <?php echo esc_html( $background_color ); ?>;
				}

				#<?php echo esc_html( $this->id ); ?> [data-columns]:before {
					content: '<?php echo esc_html( $post_columns ); ?> .grid__col.grid__col--1-of-<?php echo esc_html( $post_columns ); ?>';
				}
			</style>

			<section class="container">
				<div class="home-widget-section-title">
					<?php
						if ( $title ) {
						echo $args['before_title']; // WPCS: XSS ok.
						echo $title;  // WPCS: XSS ok.

						if ( $category_link ) :
						?>
						<a href="<?php echo esc_url( $category_link ); ?>" class="category-link"><?php esc_html_e( 'View more posts &rarr;', 'slope' ); ?></a>
						<?php
						endif;

						echo $args['after_title'];  // WPCS: XSS ok.
					}
					?>

					<?php if ( $subtitle ) : ?>
					<p class="home-widget-subtitle"><?php echo esc_attr( $subtitle ); ?></p>
					<?php endif; ?>
				</div>

				<div class="grid<?php if ( 'tile' === $layout ) echo ' grid--no-gutter'; ?>" data-columns>
					<?php
						while ( $r->have_posts() ) :
						$r->the_post();

						if ( 'tile' === $layout ) {
							get_template_part( 'template-parts/content', 'tile' );
						} else {
							get_template_part( 'template-parts/content' );
						}

						endwhile;
						?>
				</div>
			</section>

			<?php
			echo $args['after_widget']; // WPCS: XSS ok.

			endif; // End $r->have_posts() check.
		}

		/**
		 * Registers the widget with the WordPress Widget API.
		 *
		 * @return mixed
		 */
		public static function register() {
			register_widget( __CLASS__ );
		}
	}
endif;

add_action( 'widgets_init', array( 'Slope_Widget_Posts', 'register' ) );
