<?php
/**
 * Page: Products widget.
 *
 * @package Slope
 */

if ( ! is_woocommerce_activated() ) return;

if ( ! class_exists( 'Slope_Widget_Products' ) ) :
	/**
	 * Display recent posts from specific category.
	 *
	 * @package Slope
	 */
	class Slope_Widget_Products extends Slope_Widget {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->widget_id          = 'slope_products';
			$this->widget_description = esc_html__( 'Displays a list of WooCommerce products.', 'slope' );
			$this->widget_name        = esc_html__( 'Slope Section: Products', 'slope' );
			$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
			$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
			$this->settings           = array(
				'title' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Title:', 'slope' ),
				),
				'subtitle' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Subtitle:', 'slope' ),
				),
				'show' => array(
					'type'  => 'select',
					'std'   => '',
					'label' => __( 'Show', 'slope' ),
					'options' => array(
						''         => __( 'All products', 'slope' ),
						'featured' => __( 'Featured products', 'slope' ),
						'onsale'   => __( 'On-sale products', 'slope' ),
					),
				),
				'orderby' => array(
					'type'  => 'select',
					'std'   => 'date',
					'label' => __( 'Order by', 'slope' ),
					'options' => array(
						'date'   => __( 'Date', 'slope' ),
						'price'  => __( 'Price', 'slope' ),
						'rand'   => __( 'Random', 'slope' ),
						'sales'  => __( 'Sales', 'slope' ),
					),
				),
				'posts_count' => array(
					'type'  => 'number',
					'std'   => '3',
					'step'  => '1',
					'min'   => '1',
					'max'   => '1000',
					'label' => esc_html__( 'Number of posts:', 'slope' ),
				),
				'background_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#fff',
					'label' => esc_html__( 'Background Color:', 'slope' ),
				),
				'text_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#191919',
					'label' => esc_html__( 'Text Color:', 'slope' ),
				),
				'link_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#191919',
					'label' => esc_html__( 'Link Color:', 'slope' ),
				),
				'section_padding' => array(
					'type'        => 'select',
					'std'         => 'standard',
					'label'       => esc_html__( 'Section padding:', 'slope' ),
					'options'     => array(
						'100px' => esc_html__( 'Standard', 'slope' ),
						'50px'  => esc_html__( 'Half', 'slope' ),
						'0'     => esc_html__( 'None', 'slope' ),
					),
				),
			);

			parent::__construct();
		}

		/**
		 * Output the widget content on the page.
		 *
		 * @since 1.0.0
		 *
		 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current widget instance.
		 */
		public function widget( $args, $instance ) {
			// Check widget areas context.
			if ( ! is_singular( 'page' ) ) {
				echo $this->widget_areas_notice(); // WPCS: XSS ok.

				return false;
			}

			$title            = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );
			$subtitle         = isset( $instance['subtitle'] ) ? esc_attr( $instance['subtitle'] ) : false;
			$show             = isset( $instance['show'] ) ? esc_attr( $instance['show'] ) : '';
			$orderby          = isset( $instance['orderby'] ) ? esc_attr( $instance['orderby'] ) : 'date';
			$posts_count      = isset( $instance['posts_count'] ) ? esc_attr( $instance['posts_count'] ) : '3';

			$background_color = maybe_hash_hex_color( $instance['background_color'] );
			$text_color       = maybe_hash_hex_color( $instance['text_color'] );
			$section_padding  = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '100px';

			/**
			* Filters the arguments for the Posts widget.
			*
			* @see WP_Query::get_posts()
			*
			* @param array $args An array of arguments used to retrieve the recent posts.
			*/
			$query_args = array(
				'posts_per_page' => $posts_count,
				'post_status'    => 'publish',
				'post_type'      => 'product',
				'no_found_rows'  => 1,
				'meta_query'     => array(),
				'tax_query'      => array(
					'relation' => 'AND',
				),
			);

			switch ( $show ) {
				case 'featured':
					$query_args['tax_query'][] = array(
						'taxonomy' => 'product_visibility',
						'field'    => 'term_taxonomy_id',
						'terms'    => $product_visibility_term_ids['featured'],
					);
					break;
				case 'onsale':
					$product_ids_on_sale    = wc_get_product_ids_on_sale();
					$product_ids_on_sale[]  = 0;
					$query_args['post__in'] = $product_ids_on_sale;
					break;
			}

			switch ( $orderby ) {
				case 'price':
					$query_args['meta_key'] = '_price';
					$query_args['orderby']  = 'meta_value_num';
					break;
				case 'rand':
					$query_args['orderby']  = 'rand';
					break;
				case 'sales':
					$query_args['meta_key'] = 'total_sales';
					$query_args['orderby']  = 'meta_value_num';
					break;
				default:
					$query_args['orderby']  = 'date';
			}

			$products = new WP_Query( apply_filters( 'slope_products_widget_query_args', $query_args ) );

			if ( $products->have_posts() ) :
			echo $args['before_widget']; // WPCS: XSS ok.
			?>

			<style type="text/css">
				#<?php echo esc_html( $this->id ); ?> {
					background-color: <?php echo esc_html( $background_color ); ?>;
					color: <?php echo esc_html( $text_color ); ?>;
					padding: <?php echo esc_html( $section_padding ); ?> 0;
				}
			</style>

			<section class="container woocommerce">
				<div class="home-widget-section-title">
					<?php if ( $title ) echo $args['before_title'] . $title . $args['after_title']; // WPCS: XSS ok, sanitization ok. ?>

					<?php if ( $subtitle ) : ?>
					<p class="home-widget-subtitle"><?php echo esc_attr( $subtitle ); ?></p>
					<?php endif; ?>
				</div>

				<ul class="grid products">
					<?php
						while ( $products->have_posts() ) :
						$products->the_post();

						wc_get_template( 'content-product.php', array( 'show_rating' => true ) );

						endwhile;
						?>
				</ul>
			</section>

			<?php
			echo $args['after_widget']; // WPCS: XSS ok.

			wp_reset_postdata();

			endif; // End $products->have_posts() check.
		}

		/**
		 * Registers the widget with the WordPress Widget API.
		 *
		 * @return mixed
		 */
		public static function register() {
			register_widget( __CLASS__ );
		}
	}
endif;

add_action( 'widgets_init', array( 'Slope_Widget_Products', 'register' ) );
