<?php
/**
 * Page: Static Content Widget
 *
 * @package Slope
 */

if ( ! class_exists( 'Slope_Widget_Static_Content' ) ) :
	/**
	 * Display static content from an specific page.
	 *
	 * @package Slope
	 */
	class Slope_Widget_Static_Content extends Slope_Widget {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->widget_id          = 'slope_static_content';
			$this->widget_description = esc_html__( 'Displays content from a specific page.', 'slope' );
			$this->widget_name        = esc_html__( 'Slope Section: Static Content', 'slope' );
			$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
			$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
			$this->settings           = array(
				'title' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Title:', 'slope' ),
				),
				'subtitle' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Subtitle:', 'slope' ),
				),
				'page' => array(
					'type'  => 'page',
					'std'   => '',
					'label' => esc_html__( 'Select Page:', 'slope' ),
				),
				'background_image' => array(
					'type'  => 'image',
					'std'   => null,
					'label' => esc_html__( 'Background Image:', 'slope' ),
				),
				'background_opacity' => array(
					'type'  => 'number',
					'std'   => '80',
					'step'  => '10',
					'min'   => '10',
					'max'   => '100',
					'label' => esc_html__( 'Background Opacity:', 'slope' ),
				),
				'background_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#fff',
					'label' => esc_html__( 'Background Color:', 'slope' ),
				),
				'text_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#191919',
					'label' => esc_html__( 'Text Color:', 'slope' ),
				),
				'section_padding' => array(
					'type'        => 'select',
					'std'         => 'standard',
					'label'       => esc_html__( 'Section padding:', 'slope' ),
					'options'     => array(
						'100px' => esc_html__( 'Standard', 'slope' ),
						'50px'  => esc_html__( 'Half', 'slope' ),
						'0'     => esc_html__( 'None', 'slope' ),
					),
				),
				'padding_description' => array(
					'type' => 'description',
					'std'  => esc_html__( 'Determines top and bottom padding for the section.', 'slope' ),
				),
			);

			parent::__construct();
		}

		/**
		 * Output the widget content on the page.
		 *
		 * @since 1.0.0
		 *
		 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current widget instance.
		 */
		public function widget( $args, $instance ) {
			// Check widget areas context.
			if ( ! is_singular( 'page' ) ) {
				echo $this->widget_areas_notice(); // WPCS: XSS ok.

				return false;
			}

			$the_post = get_post( $instance['page'] );

			if ( ! is_object( $the_post ) ) return;

			$title              = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );
			$subtitle           = isset( $instance['subtitle'] ) ? esc_attr( $instance['subtitle'] ) : false;
			$background_image   = esc_url( $instance['background_image'] );
			$background_opacity = absint( $instance['background_opacity'] );
			$background_color   = maybe_hash_hex_color( $instance['background_color'] );
			$text_color         = maybe_hash_hex_color( $instance['text_color'] );
			$section_padding    = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '100px';

			echo $args['before_widget']; // WPCS: XSS ok.
			?>

			<?php if ( '' !== $background_image ) : ?>
			<div class="static-content-cover" style="opacity:<?php echo absint( $background_opacity ) / 100; ?>;background-image:url(<?php echo esc_url( $background_image ); ?>);"></div>
			<?php endif; ?>

			<style type="text/css">
				#<?php echo esc_html( $this->id ); ?> {
					background-color: <?php echo esc_html( $background_color ); ?>;
					color: <?php echo esc_html( $text_color ); ?>;
				}
				#<?php echo esc_html( $this->id ); ?> {
					padding: <?php echo esc_html( $section_padding ); ?> 0;
				}
			</style>

			<section class="container">
				<article id="post-<?php echo esc_attr( $the_post->ID ); ?>">
					<div class="home-widget-section-title">
						<?php if ( $title ) echo $args['before_title'] . $title . $args['after_title']; // WPCS: XSS ok, sanitization ok. ?>

						<?php if ( $subtitle ) : ?>
						<p class="home-widget-subtitle"><?php echo esc_attr( $subtitle ); ?></p>
						<?php endif; ?>
					</div>

					<div class="entry-content">
						<?php echo apply_filters( 'the_content', $the_post->post_content ); // WPCS: XSS ok. ?>
					</div>
				</article>
			</section>

			<?php
			echo $args['after_widget']; // WPCS: XSS ok.
		}

		/**
		 * Registers the widget with the WordPress Widget API.
		 *
		 * @return mixed
		 */
		public static function register() {
			register_widget( __CLASS__ );
		}
	}
endif;

add_action( 'widgets_init', array( 'Slope_Widget_Static_Content', 'register' ) );
