<?php
/**
 * Page: Testimonials Widget
 *
 * @see Jetpack <https://jetpack.com/features/design/portfolios-and-testimonials/>
 * @package Slope
 */

if ( ! class_exists( 'Slope_Widget_Testimonials' ) ) :
	/**
	 * Display a list of all site contributors who published at least one post.
	 *
	 * @package Slope
	 */
	class Slope_Widget_Testimonials extends Slope_Widget {
		/**
		 * Constructor
		 */
		public function __construct() {
			$this->widget_id          = 'slope_testimonials';
			$this->widget_description = esc_html__( 'Displays Jetpack testmonials as a slider.', 'slope' );
			$this->widget_name        = esc_html__( 'Slope Section: Testimonials', 'slope' );
			$this->widget_areas       = array( 'widget-area-page' ); // Valid widget areas.
			$this->widget_notice      = __( 'Add this widget only in "Page" widget area.', 'slope' );
			$this->settings           = array(
				'title' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Title:', 'slope' ),
				),
				'subtitle' => array(
					'type'  => 'text',
					'std'   => '',
					'label' => esc_html__( 'Subtitle:', 'slope' ),
				),
				'count' => array(
					'type'  => 'number',
					'std'   => '5',
					'step'  => '1',
					'min'   => '1',
					'max'   => '1000',
					'label' => esc_html__( 'Number of testimonials:', 'slope' ),
				),
				'background_image' => array(
					'type'  => 'image',
					'std'   => null,
					'label' => esc_html__( 'Background Image:', 'slope' ),
				),
				'background_opacity' => array(
					'type'  => 'number',
					'std'   => '80',
					'step'  => '10',
					'min'   => '10',
					'max'   => '100',
					'label' => esc_html__( 'Background Opacity:', 'slope' ),
				),
				'background_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#fff',
					'label' => esc_html__( 'Background Color:', 'slope' ),
				),
				'text_color' => array(
					'type'  => 'colorpicker',
					'std'   => '#191919',
					'label' => esc_html__( 'Text Color:', 'slope' ),
				),
				'is_centered' => array(
					'type' => 'checkbox',
					'std'  => 0,
					'label' => __( 'Center Content', 'slope' ),
				),
				'section_padding' => array(
					'type'        => 'select',
					'std'         => 'standard',
					'label'       => esc_html__( 'Section padding:', 'slope' ),
					'options'     => array(
						'100px' => esc_html__( 'Standard', 'slope' ),
						'50px'  => esc_html__( 'Half', 'slope' ),
						'0'     => esc_html__( 'None', 'slope' ),
					),
				),
				'padding_description' => array(
					'type' => 'description',
					'std'  => esc_html__( 'Determines top and bottom padding for the section.', 'slope' ),
				),
			);

			parent::__construct();

			if ( is_active_widget( false, false, $this->id_base ) || is_customize_preview() ) {
				add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
			}
		}

		/**
		 * Enqueue scripts.
		 */
		public function enqueue_scripts() {
			wp_enqueue_script( 'flexslider' );
		}

		/**
		 * Output the widget content on the page.
		 *
		 * @since 1.0.0
		 *
		 * @param array $args Display arguments including 'before_title', 'after_title', 'before_widget', and 'after_widget'.
		 * @param array $instance Settings for the current widget instance.
		 */
		public function widget( $args, $instance ) {
			// Check widget areas context.
			if ( ! is_singular( 'page' ) ) {
				echo $this->widget_areas_notice(); // WPCS: XSS ok.

				return false;
			}

			$title              = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );
			$subtitle           = isset( $instance['subtitle'] ) ? esc_attr( $instance['subtitle'] ) : false;
			$count              = isset( $instance['count'] ) ? esc_attr( $instance['count'] ) : '5';
			$background_image   = esc_url( $instance['background_image'] );
			$background_opacity = absint( $instance['background_opacity'] );
			$background_color   = maybe_hash_hex_color( $instance['background_color'] );
			$text_color         = maybe_hash_hex_color( $instance['text_color'] );
			$is_centered        = isset( $instance['is_centered'] ) ? esc_attr( $instance['is_centered'] ) : false;
			$section_padding    = isset( $instance['section_padding'] ) ? esc_attr( $instance['section_padding'] ) : '100px';

			$testimonials = new WP_Query([
				'post_type'      => 'jetpack-testimonial',
				'posts_per_page' => $count,
			]);

			// Bail, if no testmonials are found.
			if ( ! $testimonials->have_posts() ) return;

			echo $args['before_widget']; // WPCS: XSS ok.
			?>

			<?php if ( '' !== $background_image ) : ?>
			<div class="static-content-cover" style="opacity:<?php echo absint( $background_opacity ) / 100; ?>;background-image:url(<?php echo esc_url( $background_image ); ?>);"></div>
			<?php endif; ?>

			<style type="text/css">
				#<?php echo esc_html( $this->id ); ?> {
					background-color: <?php echo esc_html( $background_color ); ?>;
					color: <?php echo esc_html( $text_color ); ?>;
					padding: <?php echo esc_html( $section_padding ); ?> 0;
				}
			</style>

			<script>
				jQuery( document ).ready( function( $ ) {
					$( '#<?php echo esc_html( $this->id ); ?> .testimonials' ).flexslider({
						animation   : 'fade',
						controlNav  : false,
						directionNav: true,
						smoothHeight: true,
						prevText    : '<span class="dashicons dashicons-arrow-left-alt2"></span>',
						nextText    : '<span class="dashicons dashicons-arrow-right-alt2"></span>'
					});
				});
			</script>

			<section class="container<?php if ( $is_centered ) echo ' home-widget--centered'; ?>">
				<div class="home-widget-section-title">
					<?php if ( $title ) echo $args['before_title'] . $title . $args['after_title']; // WPCS: XSS ok, sanitization ok. ?>

					<?php if ( $subtitle ) : ?>
					<p class="home-widget-subtitle"><?php echo esc_attr( $subtitle ); ?></p>
					<?php endif; ?>
				</div>

				<div class="testimonials">
					<ul class="slides">
					<?php while ( $testimonials->have_posts() ) : ?>
						<?php $testimonials->the_post(); ?>

						<li>
							<?php
							if ( has_post_thumbnail() ) {
								the_post_thumbnail( 'thumbnail', array( 'class' => 'testimonial-avatar' ) );
							}
							?>
							<blockquote class="testimonial-content">
								<?php the_content(); ?>
								<cite><?php the_title(); ?></cite>
							</blockquote>
						</li>

					<?php endwhile; ?>
					</ul>
				</div>

			</section>

			<?php
			wp_reset_postdata();

			echo $args['after_widget']; // WPCS: XSS ok.
		}

		/**
		 * Registers the widget with the WordPress Widget API.
		 *
		 * @return mixed
		 */
		public static function register() {
			register_widget( __CLASS__ );
		}
	}
endif;

add_action( 'widgets_init', array( 'Slope_Widget_Testimonials', 'register' ) );
