<?php
/**
 * WooCommerce Compatibility File
 *
 * @link https://woocommerce.com/
 *
 * @package Slope
 */

if ( ! function_exists( 'is_woocommerce_activated' ) ) {
	/**
	 * Query whether WooCommerce is activated.
	 *
	 * @return boolean
	 */
	function is_woocommerce_activated() {
		return class_exists( 'woocommerce' ) ? true : false;
	}
}

/**
 * Check if current page is WooCommerce page.
 *
 * @return bool
 */
function slope_is_woocommerce_page() {
	if ( ! is_woocommerce_activated() ) {
		return false;
	}

	if ( is_shop() || is_cart() || is_checkout() || is_checkout_pay_page() || is_wc_endpoint_url() || is_account_page() || is_view_order_page() || is_edit_account_page() || is_order_received_page() || is_add_payment_method_page() || is_lost_password_page() ) {
		return true;
	}
	return false;
}

/**
 * Remove Jetpack's sharing buttons from WooCommerce.
 *
 * @return void
 */
function slope_woocommerce_remove_sharing_buttons() {
	if ( slope_is_woocommerce_page() ) {
		remove_filter( 'the_content', 'sharing_display', 19 );
		remove_filter( 'the_excerpt', 'sharing_display', 19 );
	}
}
add_action( 'loop_start', 'slope_woocommerce_remove_sharing_buttons', 20 );

// Bail, if WooCommerce isn't activated.
if ( ! is_woocommerce_activated() ) {
	return;
}

/**
 * WooCommerce setup function.
 *
 * @link https://docs.woocommerce.com/document/third-party-custom-theme-compatibility/
 *
 * @return void
 */
function slope_woocommerce_setup() {
	add_theme_support( 'woocommerce' );
	add_theme_support( 'wc-product-gallery-zoom' );
	add_theme_support( 'wc-product-gallery-lightbox' );
	add_theme_support( 'wc-product-gallery-slider' );
}
add_action( 'after_setup_theme', 'slope_woocommerce_setup' );

/**
 * WooCommerce specific scripts & stylesheets.
 *
 * @return void
 */
function slope_woocommerce_scripts() {
	wp_enqueue_style( 'slope-woocommerce-style', get_template_directory_uri() . '/css/woocommerce.css', array( 'dashicons' ) );
}
add_action( 'wp_enqueue_scripts', 'slope_woocommerce_scripts' );

/**
 * Add 'woocommerce-active' class to the body tag.
 *
 * @param  array $classes css classes applied to the body tag.
 * @return array $classes modified to include 'woocommerce-active' class
 */
function slope_woocommerce_active_body_class( $classes ) {
	$classes[] = 'woocommerce-active';

	if ( is_product_category() || is_product_tag() || is_shop() ) {
		$image = slope_get_thememod_value( 'fallback-shop-image' );

		if ( '' !== $image ) {
			$classes[] = 'woocommerce-category';
		}
	}
	return $classes;
}
add_filter( 'body_class', 'slope_woocommerce_active_body_class' );

/**
 * Products per page.
 *
 * @return integer number of products
 */
function slope_woocommerce_products_per_page() {
	return 12;
}
add_filter( 'loop_shop_per_page', 'slope_woocommerce_products_per_page' );

/**
 * Product gallery thumnbail columns
 *
 * @return integer number of columns
 */
function slope_woocommerce_thumbnail_columns() {
	return 4;
}
add_filter( 'woocommerce_product_thumbnails_columns', 'slope_woocommerce_thumbnail_columns' );

/**
 * Default loop columns on product archives
 *
 * @return integer products per row
 */
function slope_woocommerce_loop_columns() {
	if ( is_page_template( 'page-templates/template-widgetized.php' ) ) {
		return 3;
	}

	$custom_columns = (int) slope_get_thememod_value( 'shop-column' );
	if ( '' !== $custom_columns ) {
		return $custom_columns;
	}

	return 3;
}
add_filter( 'loop_shop_columns', 'slope_woocommerce_loop_columns' );

/**
 * Related Products Args
 *
 * @param array $args related products args.
 * @return array $args related products args
 */
function slope_woocommerce_related_products_args( $args ) {
	$args = apply_filters(
		'slope_woocommerce_related_products_args', array(
			'posts_per_page' => 3,
			'columns'        => 3,
		)
	);
	return $args;
}
add_filter( 'woocommerce_output_related_products_args', 'slope_woocommerce_related_products_args' );

if ( ! function_exists( 'slope_woocommerce_product_columns_wrapper' ) ) {
	/**
	 * Product columns wrapper
	 *
	 * @return  void
	 */
	function slope_woocommerce_product_columns_wrapper() {
		$columns = slope_woocommerce_loop_columns();
		echo '<div class="columns-' . absint( $columns ) . '">';
	}
}
add_action( 'woocommerce_before_shop_loop', 'slope_woocommerce_product_columns_wrapper', 40 );


if ( ! function_exists( 'slope_woocommerce_product_columns_wrapper_close' ) ) {
	/**
	 * Product columns wrapper close
	 *
	 * @return  void
	 */
	function slope_woocommerce_product_columns_wrapper_close() {
		echo '</div>';
	}
}
add_action( 'woocommerce_after_shop_loop', 'slope_woocommerce_product_columns_wrapper_close', 40 );

/**
 * Remove default WooCommerce wrapper
 */
remove_action( 'woocommerce_before_main_content', 'woocommerce_breadcrumb', 20, 0 );
remove_action( 'woocommerce_before_main_content', 'woocommerce_output_content_wrapper', 10 );
remove_action( 'woocommerce_after_main_content', 'woocommerce_output_content_wrapper_end', 10 );
remove_action( 'woocommerce_sidebar', 'woocommerce_get_sidebar', 10 );

if ( ! function_exists( 'slope_woocommerce_wrapper_before' ) ) {
	/**
	 * Before Content
	 * Wraps all WooCommerce content in wrappers which match the theme markup
	 *
	 * @return void
	 */
	function slope_woocommerce_wrapper_before() {
		?>
		<main id="main" class="site-main" role="main">
			<div id="primary" class="content-area">
		<?php
	}
}
add_action( 'woocommerce_before_main_content', 'slope_woocommerce_wrapper_before' );

if ( ! function_exists( 'slope_woocommerce_wrapper_after' ) ) {
	/**
	 * After Content
	 * Closes the wrapping divs
	 *
	 * @return void
	 */
	function slope_woocommerce_wrapper_after() {
		?>
			</div><!-- #primary -->

			<?php if ( is_shop() || is_post_type_archive( 'product' ) || is_tax( get_object_taxonomies( 'product' ) ) && is_active_sidebar( 'sidebar-4' ) ) : ?>
				<?php get_sidebar( 'shop' ); ?>
			<?php endif; ?>
		</main><!-- #main -->
		<?php
	}
}
add_action( 'woocommerce_after_main_content', 'slope_woocommerce_wrapper_after' );

if ( ! function_exists( 'slope_woocommerce_cart_link_fragment' ) ) {
	/**
	 * Cart Fragments
	 * Ensure cart contents update when products are added to the cart via AJAX
	 *
	 * @param array $fragments Fragments to refresh via AJAX.
	 * @return array Fragments to refresh via AJAX
	 */
	function slope_woocommerce_cart_link_fragment( $fragments ) {
		ob_start();
		slope_woocommerce_cart_link();
		$fragments['a.cart-contents'] = ob_get_clean();

		return $fragments;
	}
}
add_filter( 'woocommerce_add_to_cart_fragments', 'slope_woocommerce_cart_link_fragment' );

if ( ! function_exists( 'slope_woocommerce_cart_link' ) ) {
	/**
	 * Cart Link
	 * Displayed a link to the cart including the number of items present and the cart total
	 *
	 * @return void
	 */
	function slope_woocommerce_cart_link() {
		?>
			<a class="cart-contents" href="<?php echo esc_url( wc_get_cart_url() ); ?>" title="<?php esc_attr_e( 'View your shopping cart', 'slope' ); ?>">
				<?php
				/* translators: number of items in the mini cart. */
				?>
				<span class="dashicons dashicons-cart"></span>
				<span class="count"><?php echo wp_kses_data( WC()->cart->get_cart_contents_count() ); ?></span>
			</a>
		<?php
	}
}

if ( ! function_exists( 'slope_woocommerce_header_cart' ) ) {
	/**
	 * Display Header Cart
	 *
	 * @return void
	 */
	function slope_woocommerce_header_cart() {
		if ( is_cart() ) {
			$class = 'current-menu-item';
		} else {
			$class = '';
		}
		?>
		<ul id="site-header-cart" class="site-header-cart">
			<li class="<?php echo esc_attr( $class ); ?>">
				<?php slope_woocommerce_cart_link(); ?>
			</li>
			<li>
				<?php
				the_widget(
					'WC_Widget_Cart', array(
						'title' => '',
					)
				);
				?>
			</li>
		</ul>
		<?php
	}
}

/**
 * Define image sizes
 */
function slope_woocommerce_image_dimensions() {
	global $pagenow;

	if ( ! isset( $_GET['activated'] ) || 'themes.php' !== $pagenow ) { // WPCS: CSRF, input var ok.
		return;
	}

	$catalog = array(
		'width'  => '360',
		'height' => '400',
		'crop'   => 1,
	);

	$single = array(
		'width'  => '600',
		'height' => '600',
		'crop'   => 1,
	);

	$thumbnail = array(
		'width'  => '120',
		'height' => '120',
		'crop'   => 0,
	);

	// Image sizes.
	update_option( 'shop_catalog_image_size', $catalog );
	update_option( 'shop_single_image_size', $single );
	update_option( 'shop_thumbnail_image_size', $thumbnail );
}

add_action( 'after_switch_theme', 'slope_woocommerce_image_dimensions', 1 );

/**
 * Add background images for category archive pages.
 *
 * @return void
 */
function woocommerce_category_image() {
	if ( is_product_category() || is_product_tag() || is_shop() ) {
		// Use the fallback image value from Theme options.
		$image = slope_get_thememod_value( 'fallback-shop-image' );

		if ( '' !== $image ) {
			echo '<div class="wc-category-background viewport-wide" style="background-image:url(' . $image . ')"></div>'; // WPCS: XSS ok.
		}
	}

	return false;
}
add_action( 'woocommerce_before_main_content', 'woocommerce_category_image', 5 );

if ( ! function_exists( 'slope_is_woocommerce_page' ) ) :
	function slope_is_woocommerce_page() {
		if ( is_woocommerce() || is_cart() || is_checkout() || is_account_page() ) {
			return true;
		}
		return false;
	}
endif; // End of slope_is_woocommerce_page.

if ( ! function_exists( 'slope_woo_mobile_cart' ) ) :
	/**
	 * WooCommerce mini cart for mobile navigation.
	 */
	function slope_woo_mobile_cart() {
		?>

		<div class="woo-mobile-cart">
			<span class="dashicons dashicons-cart"></span>
			<span class="count"><?php echo wp_kses_data( WC()->cart->get_cart_contents_count() ); ?></span>
			<span class="cart-amount"><?php echo wp_kses_data( WC()->cart->get_cart_total() ); ?></span>

			<a class="cart-link" href="<?php echo esc_url( wc_get_cart_url() ); ?>" title="<?php esc_attr_e( 'View Cart', 'slope' ); ?>">
			<?php esc_html_e( 'View Cart', 'slope' ); ?>
			</a>
		</div>

		<?php
	}
endif; // End of slope_woo_mobile_cart.
